@@ -82,7 +82,7 @@ bst_file_dialog_init (BstFileDialog *self)
 		NULL);
   
   /* notebook */
-  self->notebook = g_object_new (GTK_TYPE_NOTEBOOK,
+  self->notebook = g_object_new (GXK_TYPE_NOTEBOOK,
 				 "visible", TRUE,
 				 "show_border", TRUE,
 				 "show_tabs", TRUE,
@@ -94,9 +94,6 @@ bst_file_dialog_init (BstFileDialog *self)
 				 "parent", main_box,
 				 "enable_popup", FALSE,
 				 NULL);
-  g_object_connect (self->notebook,
-		    "signal_after::switch-page", gxk_widget_viewable_changed, NULL,
-		    NULL);
   
   
   /* setup file selection widgets and add to notebook */
@@ -485,7 +482,7 @@ static gboolean
 bst_file_dialog_import_midi (BstFileDialog *self,
                              const gchar   *file_name)
 {
-  BseErrorType error = bse_project_import_midi_file (self->proxy, file_name);
+  BseErrorType error = bst_project_import_midi_file (self->proxy, file_name);
   bst_status_eprintf (error, _("Importing MIDI file `%s'"), file_name);
   return TRUE;
 }
@@ -521,7 +518,7 @@ store_bse_file (BstFileDialog *self,
           while (error == BSE_ERROR_FILE_EXISTS)
             {
               g_free (temp_file);
-              temp_file = g_strdup_printf ("%s_%u_XXXXXX", file_name, rand());
+              temp_file = g_strdup_printf ("%s.tmp%06xyXXXXXX", file_name, rand() & 0xfffffd);
               mktemp (temp_file); /* this is save, due to use of: O_CREAT | O_EXCL */
               error = bse_project_store_bse (project, super, temp_file, self_contained);
             }
@@ -737,7 +734,9 @@ bst_file_dialog_load_wave (BstFileDialog *self,
   gxk_status_printf (0, NULL, _("Loading wave `%s'"), file_name);
   error = bse_wave_repo_load_file (self->proxy, file_name);
   bst_status_eprintf (error, _("Loading wave `%s'"), file_name);
-
+  if (error)
+    sfi_error (_("Failed to load wave file \"%s\": %s"), file_name, bse_error_blurb (error));
+  
   return TRUE;
 }
 
