@@ -269,7 +269,16 @@ main (int   argc,
       /* load waves into the last project */
       if (bse_server_can_load (BSE_SERVER, argv[i]))
 	{
-	  if (!app)
+	  if (app)
+	    {
+	      SfiProxy wrepo = bse_project_get_wave_repo (app->project);
+	      gxk_status_printf (GXK_STATUS_WAIT, NULL, _("Loading \"%s\""), argv[i]);
+	      error = bse_wave_repo_load_file (wrepo, argv[i]);
+              bst_status_eprintf (error, _("Loading \"%s\""), argv[i]);
+              if (error)
+                sfi_error (_("Failed to load wave file \"%s\": %s"), argv[i], bse_error_blurb (error));
+	    }
+          else
 	    {
 	      project = bse_server_use_new_project (BSE_SERVER, "Untitled.bse");
 	      wrepo = bse_project_get_wave_repo (project);
@@ -280,21 +289,17 @@ main (int   argc,
 		  gxk_idle_show_widget (GTK_WIDGET (app));
 		  bse_item_unuse (project);
 		  gtk_widget_hide (beast_splash);
-		  continue;
 		}
-	      bse_item_unuse (project);
-	    }
-	  else
-	    {
-	      SfiProxy wrepo = bse_project_get_wave_repo (app->project);
-	      
-	      gxk_status_printf (GXK_STATUS_WAIT, NULL, _("Loading \"%s\""), argv[i]);
-	      error = bse_wave_repo_load_file (wrepo, argv[i]);
-	      bst_status_eprintf (error, _("Loading \"%s\""), argv[i]);
-	      if (!error)
-		continue;
+              else
+                {
+                  bse_item_unuse (project);
+                  sfi_error (_("Failed to load wave file \"%s\": %s"), argv[i], bse_error_blurb (error));
+                }
 	    }
+          continue;
 	}
+
+      /* load projects */
       project = bse_server_use_new_project (BSE_SERVER, argv[i]);
       error = bst_project_restore_from_file (project, argv[i]);
       
@@ -464,10 +469,10 @@ bst_early_parse_args (int    *argc_p,
       else if (strcmp ("--debug-list", argv[i]) == 0)
 	{
 	  const gchar **keys = _bst_log_debug_keys ();
-	  guint i;
+	  guint j;
 	  g_print ("debug keys: all");
-	  for (i = 0; keys[i]; i++)
-	    g_print (":%s", keys[i]);
+	  for (j = 0; keys[j]; j++)
+	    g_print (":%s", keys[j]);
 	  g_print ("\n");
 	  exit (0);
 	  argv[i] = NULL;
@@ -788,6 +793,7 @@ beast_show_about_box (void)
     "Dulmandakh Sukhbaatar",
     "Francisco Javier F. Serrador",
     "Gareth Owen",
+    "Iassen Pramatarov",
     "Ismael Andres Rubio Rojas",
     "Jorge Gonzalez",
     "Kees van den Broek",
@@ -800,6 +806,7 @@ beast_show_about_box (void)
     "Raphael Higino",
     "Robert Sedak",
     "Satoru SATOH",
+    "Steve Murphy",
     "Tino Meinen",
     "Vincent van Adrighem",
     "Xavier Conde Rueda",
