@@ -24,6 +24,31 @@
 #define intern_null_string(s,sconst)      (s ? (sconst ? g_intern_static_string : g_intern_string) (s) : NULL)
 #define intern_i18n_string(idom,s,sconst) (intern_null_string (idom && s ? dgettext (idom, s) : s, sconst))
 
+/* --- caching GxkActionCheck() --- */
+#define ACTION_CHECK( func, user_data, action_id)         (!func || action_check_stamped (func, user_data, action_id, TRUE))
+#define ACTION_CHECK_CACHED( func, user_data, action_id)  (!func || action_check_stamped (func, user_data, action_id, FALSE))
+
+static guint64 global_action_cache_stamp = 0x100000000LL;
+
+guint64
+gxk_action_inc_cache_stamp (void)
+{
+  do
+    global_action_cache_stamp += 1;
+  while ((global_action_cache_stamp & 0xffffffff) == 0);
+  return global_action_cache_stamp;
+}
+
+static inline gboolean
+action_check_stamped (GxkActionCheck acheck_func,
+                      gpointer       user_data,
+                      gulong         action_id,
+                      gboolean       alter_stamp)
+{
+  if (alter_stamp)
+    gxk_action_inc_cache_stamp();
+  return acheck_func (user_data, action_id, global_action_cache_stamp) != FALSE;
+}
 
 /* --- action class ---- */
 typedef struct {
@@ -299,7 +324,6 @@ gxk_action_list_get_action (GxkActionList          *alist,
   action->tooltip = e->action.tooltip;
   action->action_id = e->action.action_id;
   action->stock_icon = e->action.stock_icon;
-  action->options = NULL;
 }
 
 static void
@@ -337,7 +361,7 @@ gxk_action_list_force_regulate (GtkWidget *widget)
   ActionEntry *e = g_object_get_qdata (widget, quark_action_entry);
   if (e)
     {
-      gboolean sensitive = !e->klass->acheck || e->klass->acheck (e->klass->user_data, e->action.action_id);
+      gboolean sensitive = ACTION_CHECK (e->klass->acheck, e->klass->user_data, e->action.action_id);
       gboolean active = e->klass->agroup && e->klass->agroup->action_id == e->action.action_id;
       if (e->klass->agroup)
         gxk_action_group_lock (e->klass->agroup);
@@ -356,7 +380,7 @@ gxk_action_list_free (GxkActionList *alist)
   g_free (alist->entries);
   if (alist->agroup)
     g_object_unref (alist->agroup);
-   g_free (alist);
+  g_free (alist);
 }
 
 void
@@ -366,7 +390,7 @@ gxk_action_activate_callback (gconstpointer action_data)
   g_return_if_fail (e && e->klass && e->klass->ref_count > 0);
   if (!e->klass->agroup || e->klass->agroup->lock_count == 0)
     {
-      if (!e->klass->acheck || e->klass->acheck (e->klass->user_data, e->action.action_id))
+      if (ACTION_CHECK (e->klass->acheck, e->klass->user_data, e->action.action_id))
         {
           if (e->klass->agroup)
             gxk_action_group_select (e->klass->agroup, e->action.action_id);
@@ -389,89 +413,92 @@ gxk_action_activate_callback (gconstpointer action_data)
 }
 
 
-/* --- GtkWindow action sets --- */
-typedef struct _ActionSet ActionSet;
-struct _ActionSet {
+/* --- GtkWindow actions --- */
+typedef struct {
+  gpointer            client_data;
+  GxkActionClient     added_func;
+} ActionClient;
+static GQuark quark_action_clients = 0;
+
+typedef struct ActionLink ActionLink;
+struct ActionLink {
   guint          ref_count;
   gchar         *prefix;
   GtkWidget     *widget;
   GxkActionList *alist;
-  ActionSet     *next;
+  ActionLink    *next;
   GtkWidget     *toplevel;
 };
-static GQuark quark_action_sets = 0;
+static GQuark quark_action_links = 0;
 static GQuark quark_action_factories = 0;
 
 static void
-action_set_unref (ActionSet *aset)
+action_link_unref (ActionLink *alink)
 {
-  g_return_if_fail (aset->ref_count > 0);
-  aset->ref_count--;
-  if (!aset->ref_count)
+  g_return_if_fail (alink->ref_count > 0);
+  alink->ref_count--;
+  if (!alink->ref_count)
     {
-      gxk_action_list_free (aset->alist);
-      g_free (aset->prefix);
-      g_free (aset);
+      gxk_action_list_free (alink->alist);
+      g_free (alink->prefix);
+      g_free (alink);
     }
 }
 
 static void
-window_destroy_action_sets (gpointer data)
+window_destroy_action_links (gpointer data)
 {
-  ActionSet *anode = data;
+  ActionLink *anode = data;
   while (anode)
     {
-      ActionSet *aset = anode;
+      ActionLink *alink = anode;
       anode = anode->next;
-      aset->toplevel = NULL;
-      aset->next = NULL;
-      action_set_unref (aset);
+      alink->toplevel = NULL;
+      alink->next = NULL;
+      action_link_unref (alink);
     }
 }
 
 static void
-window_add_action_set (GtkWidget *window,
-                       ActionSet *aset)
+window_add_action_link (GtkWidget *window,
+                        ActionLink *alink)
 {
-  GSList *slist;
-  g_return_if_fail (GTK_IS_WIDGET (aset->widget));
-  g_return_if_fail (aset->toplevel == NULL);
-  aset->next = g_object_steal_qdata (window, quark_action_sets);
-  aset->toplevel = window;
-  aset->ref_count++;
-  g_object_set_qdata_full (window, quark_action_sets, aset, window_destroy_action_sets);
-  slist = g_object_get_qdata (window, quark_action_factories);
+  g_return_if_fail (GTK_IS_WIDGET (alink->widget));
+  g_return_if_fail (alink->toplevel == NULL);
+  alink->next = g_object_steal_qdata (window, quark_action_links);
+  alink->toplevel = window;
+  alink->ref_count++;
+  g_object_set_qdata_full (window, quark_action_links, alink, window_destroy_action_links);
+  GSList *slist = g_object_get_qdata (window, quark_action_clients);
   while (slist)
     {
-      GxkActionFactory *afactory = slist->data;
+      ActionClient *aclient = slist->data;
       slist = slist->next;
-      GXK_ACTION_FACTORY_GET_CLASS (afactory)->match_action_list (afactory, aset->prefix, aset->alist, aset->widget);
+      if (aclient->added_func)
+        aclient->added_func (aclient->client_data, (GtkWindow*) window, alink->prefix, alink->alist, alink->widget);
     }
 }
 
 static void
-window_remove_action_set (ActionSet *aset)
+window_remove_action_link (ActionLink *alink)
 {
-  GtkWidget *window = aset->toplevel;
-  ActionSet *anode, *last = NULL;
-  g_return_if_fail (GTK_IS_WIDGET (aset->toplevel));
+  GtkWidget *window = alink->toplevel;
+  g_return_if_fail (GTK_IS_WIDGET (alink->toplevel));
   
-  g_object_get_qdata (window, quark_action_sets);
-  for (anode = g_object_get_qdata (window, quark_action_sets);
-       anode;
-       last = anode, anode = last->next)
-    if (anode == aset)
+  ActionLink *last = NULL, *anode = g_object_get_qdata (window, quark_action_links);
+  for (; anode; last = anode, anode = last->next)
+    if (anode == alink)
       {
         if (last)
           last->next = anode->next;
         else
           {
-            g_object_steal_qdata (window, quark_action_sets);
-            g_object_set_qdata_full (window, quark_action_sets, anode->next, window_destroy_action_sets);
+            g_object_steal_qdata (window, quark_action_links);
+            g_object_set_qdata_full (window, quark_action_links, anode->next, window_destroy_action_links);
           }
-        aset->toplevel = NULL;
-        aset->next = NULL;
-        action_set_unref (aset);
+        alink->toplevel = NULL;
+        alink->next = NULL;
+        action_link_unref (alink);
         return;
       }
 }
@@ -508,34 +535,35 @@ static gboolean
 window_action_update_timer (gpointer data)
 {
   GDK_THREADS_ENTER ();
+  gxk_action_inc_cache_stamp();
   while (window_queue)
     {
       GtkWidget *window = g_slist_pop_head (&window_queue);
-      ActionSet *anode = g_object_get_qdata (window, quark_action_sets);
+      ActionLink *anode = g_object_get_qdata (window, quark_action_links);
       GSList *upwards = g_object_steal_qdata (window, quark_widgets_upwards);
       GSList *downwards = g_object_steal_qdata (window, quark_widgets_downwards);
       GtkWidget *last = NULL;
       gboolean needs_update = FALSE;
       for (; anode; anode = anode->next)
         {
-          ActionSet *aset = anode;
-          GxkActionList *alist = aset ? aset->alist : NULL;
+          ActionLink *alink = anode;
+          GxkActionList *alist = alink ? alink->alist : NULL;
           guint i;
-          if (!aset)
+          if (!alink)
             continue;
-          if (last != aset->widget) /* minor caching */
+          if (last != alink->widget) /* minor caching */
             {
               GSList *wnode;
-              last = aset->widget;
+              last = alink->widget;
               needs_update = FALSE;
               for (wnode = upwards; wnode; wnode = wnode->next)
-                if (check_ancestor (wnode->data, aset->widget))
+                if (check_ancestor (wnode->data, alink->widget))
                   {
                     needs_update = TRUE;
                     break;
                   }
               for (wnode = needs_update ? NULL : downwards; wnode; wnode = wnode->next)
-                if (check_ancestor (aset->widget, wnode->data))
+                if (check_ancestor (alink->widget, wnode->data))
                   {
                     needs_update = TRUE;
                     break;
@@ -548,7 +576,7 @@ window_action_update_timer (gpointer data)
               GSList *wnode;
               if (!e->widgets)
                 continue;
-              sensitive = !e->klass->acheck || e->klass->acheck (e->klass->user_data, e->action.action_id);
+              sensitive = ACTION_CHECK_CACHED (e->klass->acheck, e->klass->user_data, e->action.action_id);
               active = e->klass->agroup && e->klass->agroup->action_id == e->action.action_id;
               if (e->klass->agroup)
                 gxk_action_group_lock (e->klass->agroup);
@@ -609,12 +637,41 @@ window_queue_action_updates (GtkWidget *window,
 }
 
 
-/* --- widget action handling --- */
+
+void
+gxk_widget_update_actions_upwards (gpointer widget)
+{
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  if (GTK_IS_WINDOW (toplevel))
+    window_queue_action_updates (toplevel, widget, NULL);
+}
+
+void
+gxk_widget_update_actions_downwards (gpointer widget)
+{
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  if (GTK_IS_WINDOW (toplevel))
+    window_queue_action_updates (toplevel, NULL, widget);
+}
+
+void
+gxk_widget_update_actions (gpointer widget)
+{
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  if (GTK_IS_WINDOW (toplevel))
+    window_queue_action_updates (toplevel, widget, widget);
+}
+
+
+/* --- publishing --- */
 static GQuark  quark_widget_actions = 0;
 static GSList *publisher_list = NULL;
 
 static gboolean
-publisher_timer (gpointer data)
+action_idle_publisher (gpointer data)
 {
   GDK_THREADS_ENTER ();
   while (publisher_list)
@@ -624,13 +681,13 @@ publisher_timer (gpointer data)
       GSList *slist = g_object_get_qdata (widget, quark_widget_actions);
       while (slist)
         {
-          ActionSet *aset = slist->data;
-          if (aset->toplevel != toplevel)
+          ActionLink *alink = slist->data;
+          if (alink->toplevel != toplevel)
             {
-              if (aset->toplevel)
-                window_remove_action_set (aset);
+              if (alink->toplevel)
+                window_remove_action_link (alink);
               if (toplevel)
-                window_add_action_set (toplevel, aset);
+                window_add_action_link (toplevel, alink);
             }
           slist = slist->next;
         }
@@ -641,26 +698,26 @@ publisher_timer (gpointer data)
 }
 
 static void
-publisher_update_actions_sets (GtkWidget *widget)
+publisher_update_actions (GtkWidget *widget)
 {
   if (!g_slist_find (publisher_list, widget))
     {
       if (!publisher_list)
-        g_idle_add_full (GXK_ACTION_PRIORITY, publisher_timer, NULL, NULL);
+        g_idle_add_full (GXK_ACTION_PRIORITY, action_idle_publisher, NULL, NULL);
       publisher_list = g_slist_prepend (publisher_list, g_object_ref (widget));
     }
 }
 
 static void
-publisher_destroy_action_sets (gpointer data)
+publisher_destroy_action_links (gpointer data)
 {
   GSList *slist = data;
   while (slist)
     {
-      ActionSet *aset = g_slist_pop_head (&slist);
-      if (aset->toplevel)
-        window_remove_action_set (aset);
-      action_set_unref (aset);
+      ActionLink *alink = g_slist_pop_head (&slist);
+      if (alink->toplevel)
+        window_remove_action_link (alink);
+      action_link_unref (alink);
     }
 }
 
@@ -669,20 +726,18 @@ gxk_widget_publish_action_list (gpointer       widget,
                                 const gchar   *prefix,
                                 GxkActionList *alist)
 {
-  ActionSet *aset = g_new0 (ActionSet, 1);
-  GtkWidget *toplevel;
+  ActionLink *alink = g_new0 (ActionLink, 1);
   g_return_if_fail (GTK_IS_WIDGET (widget));
-  aset->ref_count = 1;
-  aset->widget = widget;
-  aset->prefix = g_strdup (prefix);
-  aset->alist = alist;
+  alink->ref_count = 1;
+  alink->widget = widget;
+  alink->prefix = g_strdup (prefix);
+  alink->alist = alist;
   g_object_set_qdata_full (widget, quark_widget_actions,
-                           g_slist_prepend (g_object_steal_qdata (widget, quark_widget_actions), aset),
-                           publisher_destroy_action_sets);
-  toplevel = gtk_widget_get_toplevel (widget);
-  if (!gxk_signal_handler_exists (widget, "hierarchy_changed", G_CALLBACK (publisher_update_actions_sets), NULL))
-    g_object_connect (widget, "signal_after::hierarchy-changed", publisher_update_actions_sets, NULL, NULL);
-  publisher_update_actions_sets (widget);
+                           g_slist_prepend (g_object_steal_qdata (widget, quark_widget_actions), alink),
+                           publisher_destroy_action_links);
+  if (!gxk_signal_handler_exists (widget, "hierarchy_changed", G_CALLBACK (publisher_update_actions), NULL))
+    g_object_connect (widget, "signal_after::hierarchy-changed", publisher_update_actions, NULL, NULL);
+  publisher_update_actions (widget);
   if (!gxk_signal_handler_exists (widget, "realize", G_CALLBACK (gxk_widget_update_actions), NULL))
     g_object_connect (widget, "signal_after::realize", gxk_widget_update_actions, NULL, NULL);
   gxk_widget_update_actions (widget);
@@ -693,16 +748,16 @@ gxk_widget_peek_action_widgets (gpointer                widget,
                                 const gchar            *prefix,
                                 gulong                  action_id)
 {
-  GSList *asetlist = g_object_get_qdata (widget, quark_widget_actions);
-  for (; asetlist; asetlist = asetlist->next)
+  GSList *alinklist = g_object_get_qdata (widget, quark_widget_actions);
+  for (; alinklist; alinklist = alinklist->next)
     {
-      ActionSet *aset = asetlist->data;
-      if (strcmp (aset->prefix, prefix) == 0)
+      ActionLink *alink = alinklist->data;
+      if (strcmp (alink->prefix, prefix) == 0)
         {
           guint i;
-          for (i = 0; i < aset->alist->n_entries; i++)
-            if (aset->alist->entries[i]->action.action_id == action_id)
-              return aset->alist->entries[i]->widgets;
+          for (i = 0; i < alink->alist->n_entries; i++)
+            if (alink->alist->entries[i]->action.action_id == action_id)
+              return alink->alist->entries[i]->widgets;
           return NULL;
         }
     }
@@ -714,13 +769,13 @@ gxk_widget_republish_actions (gpointer                widget,
                               const gchar            *prefix,
                               gpointer                source_widget)
 {
-  GSList *slist, *asets = g_object_get_qdata (source_widget, quark_widget_actions);
-  for (slist = asets; slist; slist = slist->next)
+  GSList *slist, *alinks = g_object_get_qdata (source_widget, quark_widget_actions);
+  for (slist = alinks; slist; slist = slist->next)
     {
-      ActionSet *aset = slist->data;
-      if (strcmp (aset->prefix, prefix) == 0)
+      ActionLink *alink = slist->data;
+      if (strcmp (alink->prefix, prefix) == 0)
         {
-          gxk_widget_publish_action_list (widget, prefix, gxk_action_list_copy (aset->alist));
+          gxk_widget_publish_action_list (widget, prefix, gxk_action_list_copy (alink->alist));
           return;
         }
     }
@@ -793,33 +848,47 @@ gxk_widget_publish_translated (gpointer                widget,
 }
 
 void
-gxk_widget_update_actions_upwards (gpointer widget)
+gxk_window_add_action_client (GtkWindow              *window,
+                              GxkActionClient         added_func,
+                              gpointer                client_data)
 {
-  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  if (GTK_IS_WINDOW (toplevel))
-    window_queue_action_updates (toplevel, widget, NULL);
+  g_return_if_fail (GTK_IS_WINDOW (window));
+  g_return_if_fail (client_data != NULL);
+  ActionClient *aclient = g_new0 (ActionClient, 1);
+  aclient->client_data = client_data;
+  aclient->added_func = added_func;
+  GSList *slist = g_object_get_qdata (window, quark_action_clients);
+  g_object_set_qdata (window, quark_action_clients, g_slist_prepend (slist, aclient));
+  ActionLink *alink;
+  for (alink = g_object_get_qdata (window, quark_action_links); alink; alink = alink->next)
+    aclient->added_func (aclient->client_data, window, alink->prefix, alink->alist, alink->widget);
 }
 
 void
-gxk_widget_update_actions_downwards (gpointer widget)
+gxk_window_remove_action_client (GtkWindow              *window,
+                                 gpointer                client_data)
 {
-  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  if (GTK_IS_WINDOW (toplevel))
-    window_queue_action_updates (toplevel, NULL, widget);
+  g_return_if_fail (GTK_IS_WINDOW (window));
+  g_return_if_fail (client_data != NULL);
+  GSList *last = NULL, *slist = g_object_get_qdata (window, quark_action_clients);
+  while (slist)
+    {
+      ActionClient *aclient = slist->data;
+      if (aclient->client_data == client_data)
+        {
+          if (last)
+            last->next = slist->next;
+          else
+            g_object_set_qdata (window, quark_action_clients, slist->next);
+          g_free (aclient);
+          return;
+        }
+      last = slist;
+      slist = last->next;
+    }
+  g_warning ("failed to remove action client (%p) from GtkWindow (%p)", client_data, window);
 }
 
-void
-gxk_widget_update_actions (gpointer widget)
-{
-  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  if (GTK_IS_WINDOW (toplevel))
-    window_queue_action_updates (toplevel, widget, widget);
-}
-
-
 /* --- action group --- */
 static gulong action_group_signal_changed = 0;
 
@@ -909,60 +978,17 @@ gxk_action_group_dispose (GxkActionGroup *self)
 }
 
 
-/* --- action factory --- */
-GType
-gxk_action_factory_get_type (void)
-{
-  static GType type = 0;
-  if (!type)
-    {
-      static const GTypeInfo type_info = {
-        sizeof (GxkActionFactoryClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) NULL,
-        NULL,   /* class_finalize */
-        NULL,   /* class_data */
-        sizeof (GxkActionFactory),
-        0,      /* n_preallocs */
-        (GInstanceInitFunc) NULL,
-      };
-      type = g_type_register_static (G_TYPE_OBJECT, "GxkActionFactory", &type_info, 0);
-    }
-  return type;
-}
-
-void
-gxk_window_add_action_factory (GtkWindow        *window,
-                               GxkActionFactory *afactory)
-{
-  GSList *slist = g_object_get_qdata (window, quark_action_factories);
-  ActionSet *aset;
-  g_object_set_qdata (window, quark_action_factories, g_slist_prepend (slist, afactory));
-  for (aset = g_object_get_qdata (window, quark_action_sets); aset; aset = aset->next)
-    GXK_ACTION_FACTORY_GET_CLASS (afactory)->match_action_list (afactory, aset->prefix, aset->alist, aset->widget);
-}
-
-void
-gxk_window_remove_action_factory (GtkWindow        *window,
-                                  GxkActionFactory *afactory)
-{
-  GSList *slist = g_object_get_qdata (window, quark_action_factories);
-  slist = g_slist_remove (slist, afactory);
-  g_object_set_qdata (window, quark_action_factories, slist);
-}
-
-
 /* --- initialization --- */
 void
-_gxk_init_actions (void)
+gxk_init_actions (void)
 {
   g_assert (action_class_ht == NULL);
   action_class_ht = g_hash_table_new (action_class_hash, action_class_equals);
-  quark_action_sets = g_quark_from_static_string ("GxkWindow-action-sets");
+  quark_action_links = g_quark_from_static_string ("gxk-action-links");
   quark_action_factories = g_quark_from_static_string ("GxkActionFactory-list");
-  quark_action_entry = g_quark_from_static_string ("GxkAction-entry");
-  quark_widget_actions = g_quark_from_static_string ("GxkWidget-action-sets");
-  quark_widgets_upwards = g_quark_from_static_string ("GxkWindow-widgets-upwards");
-  quark_widgets_downwards = g_quark_from_static_string ("GxkWindow-widgets-downwards");
+  quark_action_entry = g_quark_from_static_string ("gxk-action-entry");
+  quark_action_clients = g_quark_from_static_string ("gxk-action-clients");
+  quark_widget_actions = g_quark_from_static_string ("gxk-action-widget-action-links");
+  quark_widgets_upwards = g_quark_from_static_string ("gxk-action-widgets-upwards");
+  quark_widgets_downwards = g_quark_from_static_string ("gxk-action-widgets-downwards");
 }
