@@ -28,22 +28,21 @@ G_BEGIN_DECLS
 
 
 /* --- structures --- */
-typedef gboolean (*GxkActionCheck)      (gpointer        user_data,
-                                         gulong          action_id);
-typedef void     (*GxkActionExec)       (gpointer        user_data,
-                                         gulong          action_id);
-typedef        GObject        GxkActionFactory; /* prototyped */
+typedef gboolean (*GxkActionCheck)        (gpointer        user_data,
+                                           gulong          action_id,
+                                           guint64         action_stamp);
+typedef void     (*GxkActionExec)         (gpointer        user_data,
+                                           gulong          action_id);
 typedef struct GxkActionGroup GxkActionGroup;   /* prototyped */
 typedef struct GxkActionList  GxkActionList;
 typedef struct {
-  const gchar  *key;            /* untranslated name */
+  const gchar  *key;            /* untranslated name (used for accel paths) */
   gconstpointer action_data;    /* for gxk_action_activate_callback() */
   const gchar  *name;
   const gchar  *accelerator;
   const gchar  *tooltip;
   gulong        action_id;
   const gchar  *stock_icon;     /* stock_id for the icon or NULL */
-  const gchar  *options;
 } GxkAction;
 
 typedef struct {
@@ -54,8 +53,8 @@ typedef struct {
   const gchar  *stock_icon;     /* stock_id for the icon */
 } GxkStockAction;
 
-
 /* --- public API --- */
+guint64         gxk_action_inc_cache_stamp      (void);
 GxkActionList*  gxk_action_list_create          (void);
 GxkActionList*  gxk_action_list_create_grouped  (GxkActionGroup         *agroup);
 void            gxk_action_list_add_actions     (GxkActionList          *alist,
@@ -89,10 +88,11 @@ void            gxk_action_list_regulate_widget (GxkActionList          *alist,
 void            gxk_action_list_force_regulate  (GtkWidget              *widget);
 void            gxk_action_list_free            (GxkActionList          *alist);
 void            gxk_action_activate_callback    (gconstpointer          action_data);
-void            gxk_window_add_action_factory   (GtkWindow              *window,
-                                                 GxkActionFactory       *afactory);
-void            gxk_window_remove_action_factory(GtkWindow              *window,
-                                                 GxkActionFactory       *afactory);
+void      gxk_widget_update_actions_upwards     (gpointer                widget);
+void      gxk_widget_update_actions_downwards   (gpointer                widget);
+void      gxk_widget_update_actions             (gpointer                widget);
+
+/* --- publishing --- */
 void      gxk_widget_publish_action_list        (gpointer                widget,
                                                  const gchar            *prefix,
                                                  GxkActionList          *alist);
@@ -138,9 +138,16 @@ void      gxk_widget_publish_grouped_translated (gpointer                widget,
 void      gxk_widget_republish_actions          (gpointer                widget,
                                                  const gchar            *prefix,
                                                  gpointer                source_widget);
-void      gxk_widget_update_actions_upwards     (gpointer                widget);
-void      gxk_widget_update_actions_downwards   (gpointer                widget);
-void      gxk_widget_update_actions             (gpointer                widget);
+typedef void  (*GxkActionClient)                (gpointer                client_data,
+                                                 GtkWindow              *window,
+                                                 const gchar            *prefix,
+                                                 GxkActionList          *action_list,
+                                                 GtkWidget              *publisher);
+void      gxk_window_add_action_client          (GtkWindow              *window,
+                                                 GxkActionClient         added_func,
+                                                 gpointer                client_data);
+void      gxk_window_remove_action_client       (GtkWindow              *window,
+                                                 gpointer                client_data);
 
 
 /* --- action groups --- */
@@ -170,23 +177,6 @@ void            gxk_action_group_dispose        (GxkActionGroup        *agroup);
 GxkActionGroup* gxk_action_toggle_new           (void);
 
 
-/* --- action factory --- */
-#define GXK_TYPE_ACTION_FACTORY              (gxk_action_factory_get_type ())
-#define GXK_ACTION_FACTORY(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GXK_TYPE_ACTION_FACTORY, GxkActionFactory))
-#define GXK_ACTION_FACTORY_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GXK_TYPE_ACTION_FACTORY, GxkActionFactoryClass))
-#define GXK_IS_ACTION_FACTORY(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), GXK_TYPE_ACTION_FACTORY))
-#define GXK_IS_ACTION_FACTORY_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GXK_TYPE_ACTION_FACTORY))
-#define GXK_ACTION_FACTORY_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS ((object), GXK_TYPE_ACTION_FACTORY, GxkActionFactoryClass))
-typedef struct {
-  GObjectClass parent_class;
-  void  (*match_action_list)    (GxkActionFactory       *self,
-                                 const gchar            *prefix,
-                                 GxkActionList          *alist,
-                                 GtkWidget              *publisher);
-} GxkActionFactoryClass;
-GType   gxk_action_factory_get_type     (void);
-
-
 G_END_DECLS
 
 #endif /* __GXK_ACTION_H__ */
