@@ -61,6 +61,8 @@ static void             navigate_reload         (GtkWidget      *sctext);
 static void             navigate_index          (GtkWidget      *sctext);
 static void             navigate_find           (GtkWidget      *sctext);
 static void             navigate_goto           (GtkWidget      *sctext);
+static void             navigate_link           (GtkWidget      *sctext,
+                                                 const gchar    *uri);
 static void             text_buffer_add_error   (GtkTextBuffer  *tbuffer,
                                                  const gchar    *format,
                                                  ...) G_GNUC_PRINTF (2, 3);
@@ -1268,11 +1270,14 @@ scroll_text_patchup_size_request (GtkWidget      *scwin,
                                   GtkRequisition *requisition,
                                   GtkWidget      *sctext)
 {
+  GxkScrollTextFlags flags = g_object_get_long (sctext, "GxkScrollTextFlags");
   if (!GTK_WIDGET_MAPPED (scwin))
     {
       /* provide initial size */
-      requisition->width += 220;
-      requisition->height += 100;
+      if (!(flags & GXK_SCROLL_TEXT_HFIXED))
+        requisition->width += 220;
+      if (!(flags & GXK_SCROLL_TEXT_VFIXED))
+        requisition->height += 100;
     }
 }
 
@@ -1301,6 +1306,8 @@ tnav_update_vpos (GtkAdjustment  *a,
  @* GXK_SCROLL_TEXT_NAVIGATABLE - add a navigation bar and allow
  * the user to navigate through clickable links;
  @* GXK_SCROLL_TEXT_EDITABLE - permit modifications of the text;
+ @* GXK_SCROLL_TEXT_HFIXED - make horizontal dimension unscrollable
+ @* GXK_SCROLL_TEXT_VFIXED - make vertical dimension unscrollable
  @* GXK_SCROLL_TEXT_WIDGET_LOOK - this is a combination of flags
  * to adjust the scroll text to look like an ordinary #GtkLabel,
  * which amounts to using a sans serif font, normal widget
@@ -1320,6 +1327,7 @@ gxk_scroll_text_create (GxkScrollTextFlags flags,
   sctext = g_object_new (GTK_TYPE_VBOX,
                          "visible", TRUE,
                          NULL);
+  g_object_set_long (sctext, "GxkScrollTextFlags", flags);
   /* navigation toolbar */
   if (flags & GXK_SCROLL_TEXT_NAVIGATABLE)
     {
@@ -1332,8 +1340,8 @@ gxk_scroll_text_create (GxkScrollTextFlags flags,
   /* scrollable text area */
   scwin = g_object_new (GTK_TYPE_SCROLLED_WINDOW,
                         "visible", TRUE,
-                        "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
-                        "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
+                        "hscrollbar_policy", (flags & GXK_SCROLL_TEXT_HFIXED) ? GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC,
+                        "vscrollbar_policy", (flags & GXK_SCROLL_TEXT_VFIXED) ? GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC,
                         NULL);
   g_signal_connect_after (scwin, "size_request", G_CALLBACK (scroll_text_patchup_size_request), sctext);
   gtk_box_pack_start (GTK_BOX (sctext), scwin, TRUE, TRUE, 0);
@@ -1351,7 +1359,7 @@ gxk_scroll_text_create (GxkScrollTextFlags flags,
       TextNavigation *tnav = navigation_from_sctext (sctext);
       tnav->vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scwin));
       g_object_connect (tnav->vadjustment, "signal::value_changed", tnav_update_vpos, tnav, NULL);
-      g_signal_connect_swapped (tbuffer, "custom-activate", G_CALLBACK (gxk_scroll_text_advance), sctext);
+      g_signal_connect_swapped (tbuffer, "custom-activate", G_CALLBACK (navigate_link), sctext);
       tnav->backb = gxk_radget_find (toolbar, "back-button");
       g_object_connect (g_object_ref (tnav->backb), "swapped_signal::clicked", navigate_back, sctext, NULL);
       gtk_widget_set_sensitive (tnav->backb, FALSE);
@@ -2297,3 +2305,16 @@ navigate_goto (GtkWidget *sctext)
   if (text && text[0])
     gxk_scroll_text_enter (sctext, text);
 }
+
+static void
+navigate_link (GtkWidget   *sctext,
+               const gchar *uri)
+{
+  if (strncmp (uri, "ftp:", 4) == 0 ||
+      strncmp (uri, "http:", 5) == 0 ||
+      strncmp (uri, "https:", 6) == 0 ||
+      strncmp (uri, "mailto:", 7) == 0)
+    gxk_show_url (uri);
+  else
+    gxk_scroll_text_advance (sctext, uri);
+}
