@@ -50,6 +50,8 @@ void            gxk_widget_request_vclient_width   (GtkWidget       *widget,
                                                     GtkWidget       *client);
 gboolean        gxk_widget_has_ancestor            (gpointer         widget,
                                                     gpointer         ancestor);
+void            gxk_menu_set_active                (GtkMenu         *menu,
+                                                    GtkWidget       *child);
 void            gxk_widget_regulate                (GtkWidget       *widget,
                                                     gboolean         sensitive,
                                                     gboolean         active);
@@ -67,7 +69,18 @@ void            gxk_menu_attach_as_popup_with_func (GtkMenu         *menu,
 void            gxk_menu_popup                     (GtkMenu         *menu,
                                                     gint             x,
                                                     gint             y,
-                                                    gboolean         push_in,
+                                                    guint            mouse_button,
+                                                    guint32          time);
+void            gxk_menu_popup_pushable            (GtkMenu         *menu,
+                                                    gint             x,
+                                                    gint             y,
+                                                    gint             pushed_x,
+                                                    gint             pushed_y,
+                                                    guint            mouse_button,
+                                                    guint32          time);
+void            gxk_menu_popup_pushed_in           (GtkMenu         *menu,
+                                                    gint             pushed_x,
+                                                    gint             pushed_y,
                                                     guint            mouse_button,
                                                     guint32          time);
 GtkWidget*      gxk_widget_find_level_ordered      (GtkWidget       *toplevel,
@@ -147,8 +160,9 @@ gchar*       gxk_factory_path_unescape_uline (const gchar    *path);
 
 
 /* --- Gtk+ Utilities --- */
-void	 gxk_widget_viewable_changed	(GtkWidget		*widget);
-gboolean gxk_widget_viewable		(GtkWidget		*widget);
+void	 gxk_widget_viewable_changed	        (GtkWidget   *widget);
+gboolean gxk_widget_viewable		        (GtkWidget   *widget);
+void     gxk_widget_attached_hierarchy_changed  (GtkWidget   *widget);
 
 
 /* --- Gdk convenience --- */
