@@ -82,11 +82,12 @@ static GEnumValue bse_error_type_values[] = {
   { BSE_ERROR_NONE, "BSE_ERROR_NONE", "none" },
   { BSE_ERROR_INTERNAL, "BSE_ERROR_INTERNAL", "internal" },
   { BSE_ERROR_UNKNOWN, "BSE_ERROR_UNKNOWN", "unknown" },
-  { BSE_ERROR_FILE_IO, "BSE_ERROR_FILE_IO", "file-io" },
-  { BSE_ERROR_FILE_PERMS, "BSE_ERROR_FILE_PERMS", "file-perms" },
+  { BSE_ERROR_IO, "BSE_ERROR_IO", "io" },
+  { BSE_ERROR_PERMS, "BSE_ERROR_PERMS", "perms" },
   { BSE_ERROR_FILE_BUSY, "BSE_ERROR_FILE_BUSY", "file-busy" },
   { BSE_ERROR_FILE_EXISTS, "BSE_ERROR_FILE_EXISTS", "file-exists" },
   { BSE_ERROR_FILE_EOF, "BSE_ERROR_FILE_EOF", "file-eof" },
+  { BSE_ERROR_FILE_EMPTY, "BSE_ERROR_FILE_EMPTY", "file-empty" },
   { BSE_ERROR_FILE_NOT_FOUND, "BSE_ERROR_FILE_NOT_FOUND", "file-not-found" },
   { BSE_ERROR_FILE_IS_DIR, "BSE_ERROR_FILE_IS_DIR", "file-is-dir" },
   { BSE_ERROR_FILE_OPEN_FAILED, "BSE_ERROR_FILE_OPEN_FAILED", "file-open-failed" },
@@ -101,12 +102,18 @@ static GEnumValue bse_error_type_values[] = {
   { BSE_ERROR_NO_SEEK_INFO, "BSE_ERROR_NO_SEEK_INFO", "no-seek-info" },
   { BSE_ERROR_NO_DATA, "BSE_ERROR_NO_DATA", "no-data" },
   { BSE_ERROR_DATA_CORRUPT, "BSE_ERROR_DATA_CORRUPT", "data-corrupt" },
+  { BSE_ERROR_WRONG_N_CHANNELS, "BSE_ERROR_WRONG_N_CHANNELS", "wrong-n-channels" },
   { BSE_ERROR_FORMAT_INVALID, "BSE_ERROR_FORMAT_INVALID", "format-invalid" },
   { BSE_ERROR_FORMAT_UNKNOWN, "BSE_ERROR_FORMAT_UNKNOWN", "format-unknown" },
+  { BSE_ERROR_DATA_UNMATCHED, "BSE_ERROR_DATA_UNMATCHED", "data-unmatched" },
   { BSE_ERROR_TEMP, "BSE_ERROR_TEMP", "temp" },
   { BSE_ERROR_WAVE_NOT_FOUND, "BSE_ERROR_WAVE_NOT_FOUND", "wave-not-found" },
   { BSE_ERROR_CODEC_FAILURE, "BSE_ERROR_CODEC_FAILURE", "codec-failure" },
   { BSE_ERROR_UNIMPLEMENTED, "BSE_ERROR_UNIMPLEMENTED", "unimplemented" },
+  { BSE_ERROR_INVALID_PROPERTY, "BSE_ERROR_INVALID_PROPERTY", "invalid-property" },
+  { BSE_ERROR_INVALID_MIDI_CONTROL, "BSE_ERROR_INVALID_MIDI_CONTROL", "invalid-midi-control" },
+  { BSE_ERROR_PARSE_ERROR, "BSE_ERROR_PARSE_ERROR", "parse-error" },
+  { BSE_ERROR_SPAWN, "BSE_ERROR_SPAWN", "spawn" },
   { BSE_ERROR_DEVICE_NOT_AVAILABLE, "BSE_ERROR_DEVICE_NOT_AVAILABLE", "device-not-available" },
   { BSE_ERROR_DEVICE_ASYNC, "BSE_ERROR_DEVICE_ASYNC", "device-async" },
   { BSE_ERROR_DEVICE_BUSY, "BSE_ERROR_DEVICE_BUSY", "device-busy" },
@@ -133,10 +140,6 @@ static GEnumValue bse_error_type_values[] = {
   { BSE_ERROR_PROC_PARAM_INVAL, "BSE_ERROR_PROC_PARAM_INVAL", "proc-param-inval" },
   { BSE_ERROR_PROC_EXECUTION, "BSE_ERROR_PROC_EXECUTION", "proc-execution" },
   { BSE_ERROR_PROC_ABORT, "BSE_ERROR_PROC_ABORT", "proc-abort" },
-  { BSE_ERROR_INVALID_PROPERTY, "BSE_ERROR_INVALID_PROPERTY", "invalid-property" },
-  { BSE_ERROR_INVALID_MIDI_CONTROL, "BSE_ERROR_INVALID_MIDI_CONTROL", "invalid-midi-control" },
-  { BSE_ERROR_PARSE_ERROR, "BSE_ERROR_PARSE_ERROR", "parse-error" },
-  { BSE_ERROR_SPAWN, "BSE_ERROR_SPAWN", "spawn" },
   { BSE_ERROR_NO_ENTRY, "BSE_ERROR_NO_ENTRY", "no-entry" },
   { BSE_ERROR_NO_EVENT, "BSE_ERROR_NO_EVENT", "no-event" },
   { BSE_ERROR_NO_TARGET, "BSE_ERROR_NO_TARGET", "no-target" },
