@@ -576,6 +576,7 @@ typedef struct _BseNoteSequence BseNoteSequence;
 typedef struct _BseIcon BseIcon;
 typedef struct _BseCategory BseCategory;
 typedef struct _BseDot BseDot;
+typedef struct _BsePartLink BsePartLink;
 typedef struct _BseTrackPart BseTrackPart;
 typedef struct _BseSongTiming BseSongTiming;
 typedef struct _BseSampleFileInfo BseSampleFileInfo;
@@ -595,6 +596,7 @@ typedef struct _BsePartNoteSeq BsePartNoteSeq;
 typedef struct _BsePartControlSeq BsePartControlSeq;
 typedef struct _BseCategorySeq BseCategorySeq;
 typedef struct _BseDotSeq BseDotSeq;
+typedef struct _BsePartLinkSeq BsePartLinkSeq;
 typedef struct _BseTrackPartSeq BseTrackPartSeq;
 typedef struct _BseThreadInfoSeq BseThreadInfoSeq;
 
@@ -636,6 +638,10 @@ struct _BseDotSeq {
   guint n_dots;
   BseDot** dots;
 };
+struct _BsePartLinkSeq {
+  guint n_plinks;
+  BsePartLink** plinks;
+};
 struct _BseTrackPartSeq {
   guint n_tparts;
   BseTrackPart** tparts;
@@ -704,6 +710,12 @@ struct _BseDot {
   SfiReal x;
   SfiReal y;
 };
+struct _BsePartLink {
+  BseTrack* track;
+  SfiInt tick;
+  BsePart* part;
+  SfiInt duration;
+};
 struct _BseTrackPart {
   SfiInt tick;
   BsePart* part;
@@ -822,6 +834,13 @@ BseDot* bse_dot_from_rec (SfiRec *sfi_rec);
 SfiRec *bse_dot_to_rec (BseDot* rec);
 void bse_dot_free (BseDot* rec);
 
+SfiRecFields bse_part_link_get_fields (void);
+BsePartLink* bse_part_link_new (void);
+BsePartLink* bse_part_link_copy_shallow (BsePartLink* rec);
+BsePartLink* bse_part_link_from_rec (SfiRec *sfi_rec);
+SfiRec *bse_part_link_to_rec (BsePartLink* rec);
+void bse_part_link_free (BsePartLink* rec);
+
 SfiRecFields bse_track_part_get_fields (void);
 BseTrackPart* bse_track_part_new (void);
 BseTrackPart* bse_track_part_copy_shallow (BseTrackPart* rec);
@@ -955,6 +974,15 @@ SfiSeq *bse_dot_seq_to_seq (BseDotSeq* seq);
 void bse_dot_seq_resize (BseDotSeq* seq, guint new_size);
 void bse_dot_seq_free (BseDotSeq* seq);
 
+GParamSpec* bse_part_link_seq_get_element (void);
+BsePartLinkSeq* bse_part_link_seq_new (void);
+void bse_part_link_seq_append (BsePartLinkSeq* seq, BsePartLink* element);
+BsePartLinkSeq* bse_part_link_seq_copy_shallow (BsePartLinkSeq* seq);
+BsePartLinkSeq* bse_part_link_seq_from_seq (SfiSeq *sfi_seq);
+SfiSeq *bse_part_link_seq_to_seq (BsePartLinkSeq* seq);
+void bse_part_link_seq_resize (BsePartLinkSeq* seq, guint new_size);
+void bse_part_link_seq_free (BsePartLinkSeq* seq);
+
 GParamSpec* bse_track_part_seq_get_element (void);
 BseTrackPartSeq* bse_track_part_seq_new (void);
 void bse_track_part_seq_append (BseTrackPartSeq* seq, BseTrackPart* element);
@@ -998,6 +1026,7 @@ extern GType bse__type_id__note_sequence;
 extern GType bse__type_id__icon;
 extern GType bse__type_id__category;
 extern GType bse__type_id__dot;
+extern GType bse__type_id__part_link;
 extern GType bse__type_id__track_part;
 extern GType bse__type_id__song_timing;
 extern GType bse__type_id__sample_file_info;
@@ -1017,6 +1046,7 @@ extern GType bse__type_id__part_note_seq;
 extern GType bse__type_id__part_control_seq;
 extern GType bse__type_id__category_seq;
 extern GType bse__type_id__dot_seq;
+extern GType bse__type_id__part_link_seq;
 extern GType bse__type_id__track_part_seq;
 extern GType bse__type_id__thread_info_seq;
 
@@ -1039,6 +1069,7 @@ extern GType bse__type_id__thread_info_seq;
 #define BSE_TYPE_ICON		(bse__type_id__icon)
 #define BSE_TYPE_CATEGORY		(bse__type_id__category)
 #define BSE_TYPE_DOT		(bse__type_id__dot)
+#define BSE_TYPE_PART_LINK		(bse__type_id__part_link)
 #define BSE_TYPE_TRACK_PART		(bse__type_id__track_part)
 #define BSE_TYPE_SONG_TIMING		(bse__type_id__song_timing)
 #define BSE_TYPE_SAMPLE_FILE_INFO		(bse__type_id__sample_file_info)
@@ -1058,6 +1089,7 @@ extern GType bse__type_id__thread_info_seq;
 #define BSE_TYPE_PART_CONTROL_SEQ		(bse__type_id__part_control_seq)
 #define BSE_TYPE_CATEGORY_SEQ		(bse__type_id__category_seq)
 #define BSE_TYPE_DOT_SEQ		(bse__type_id__dot_seq)
+#define BSE_TYPE_PART_LINK_SEQ		(bse__type_id__part_link_seq)
 #define BSE_TYPE_TRACK_PART_SEQ		(bse__type_id__track_part_seq)
 #define BSE_TYPE_THREAD_INFO_SEQ		(bse__type_id__thread_info_seq)
 
