@@ -19,6 +19,7 @@
 
 #include "bsemidievent.h"	/* BSE_MIDI_MAX_CHANNELS */
 #include "bsemidivoice.h"
+#include "bsemidireceiver.h"
 #include "bsecontextmerger.h"
 #include "bsesubsynth.h"
 #include "bsepcmoutput.h"
@@ -29,8 +30,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 
-#include "./icons/snet.c"
-
 
 /* --- parameters --- */
 enum
@@ -39,7 +38,7 @@ enum
   PROP_MIDI_CHANNEL,
   PROP_N_VOICES,
   PROP_SNET,
-  PROP_POST_NET,
+  PROP_PNET,
   PROP_VOLUME_f,
   PROP_VOLUME_dB,
   PROP_VOLUME_PERC,
@@ -59,12 +58,9 @@ static void         bse_midi_synth_get_property        (GObject           *msynt
                                                         guint              param_id,
                                                         GValue            *value,
                                                         GParamSpec        *pspec);
-static BseItemSeq*  bse_midi_synth_list_items          (BseItem           *item,
-                                                        guint              param_id,
-                                                        GParamSpec        *pspec);
 static void         bse_midi_synth_context_create      (BseSource         *source,
                                                         guint              context_handle,
-                                                        GslTrans          *trans);
+                                                        BseTrans          *trans);
 static void         bse_misi_synth_update_midi_channel (BseMidiSynth      *self);
 
 
@@ -100,80 +96,16 @@ BSE_BUILTIN_TYPE (BseMidiSynth)
   return midi_synth_type;
 }
 
-static void
-bse_midi_synth_class_init (BseMidiSynthClass *class)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
-  BseItemClass *item_class = BSE_ITEM_CLASS (class);
-  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
-  
-  parent_class = g_type_class_peek_parent (class);
-  
-  gobject_class->set_property = bse_midi_synth_set_property;
-  gobject_class->get_property = bse_midi_synth_get_property;
-  gobject_class->finalize = bse_midi_synth_finalize;
-  
-  item_class->list_items = bse_midi_synth_list_items;
-  
-  source_class->context_create = bse_midi_synth_context_create;
-  
-  bse_object_class_add_param (object_class, "MIDI Instrument",
-			      PROP_MIDI_CHANNEL,
-			      sfi_pspec_int ("midi_channel", "MIDI Channel", NULL,
-					     1, 1, BSE_MIDI_MAX_CHANNELS, 1,
-					     SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale:skip-default"));
-  bse_object_class_add_param (object_class, "MIDI Instrument",
-			      PROP_N_VOICES,
-			      sfi_pspec_int ("n_voices", "Max Voices", "Maximum number of voices for simultaneous playback",
-					     16, 1, 256, 1,
-					     SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale"));
-  bse_object_class_add_param (object_class, "MIDI Instrument",
-			      PROP_SNET,
-			      bse_param_spec_object ("snet", "Synthesis Network", "The MIDI instrument synthesis network",
-						     BSE_TYPE_CSYNTH, SFI_PARAM_STANDARD));
-  bse_object_class_add_param (object_class, "MIDI Instrument",
-                              PROP_POST_NET,
-                              bse_param_spec_object ("pnet", "Custom Postprocess Net",
-                                                     "Synthesis network to postprocess MIDI sound",
-                                                     BSE_TYPE_CSYNTH,
-                                                     SFI_PARAM_STANDARD));
-  bse_object_class_add_param (object_class, "Adjustments",
-			      PROP_VOLUME_f,
-			      sfi_pspec_real ("volume_f", "Master [float]", NULL,
-					      bse_dB_to_factor (BSE_DFL_MASTER_VOLUME_dB),
-					      0, bse_dB_to_factor (BSE_MAX_VOLUME_dB), 0.1,
-					      SFI_PARAM_STORAGE));
-  bse_object_class_add_param (object_class, "Adjustments",
-			      PROP_VOLUME_dB,
-			      sfi_pspec_real ("volume_dB", "Master [dB]", NULL,
-					      BSE_DFL_MASTER_VOLUME_dB,
-					      BSE_MIN_VOLUME_dB, BSE_MAX_VOLUME_dB,
-					      BSE_GCONFIG (step_volume_dB),
-					      SFI_PARAM_GUI ":dial"));
-  bse_object_class_add_param (object_class, "Adjustments",
-			      PROP_VOLUME_PERC,
-			      sfi_pspec_int ("volume_perc", "Master [%]", NULL,
-					     bse_dB_to_factor (BSE_DFL_MASTER_VOLUME_dB) * 100,
-					     0, bse_dB_to_factor (BSE_MAX_VOLUME_dB) * 100, 1,
-					     SFI_PARAM_GUI ":dial"));
-  bse_object_class_add_param (object_class, "Playback Settings",
-			      PROP_AUTO_ACTIVATE,
-			      sfi_pspec_bool ("auto_activate", NULL, NULL,
-					      TRUE, /* change default */
-					      /* override parent property */ 0));
-}
-
 static void
 bse_midi_synth_init (BseMidiSynth *self)
 {
   BseSNet *snet = BSE_SNET (self);
 
   BSE_OBJECT_UNSET_FLAGS (self, BSE_SNET_FLAG_USER_SYNTH);
-  BSE_OBJECT_SET_FLAGS (self, BSE_SUPER_FLAG_NEEDS_CONTEXT | BSE_SUPER_FLAG_NEEDS_SEQUENCER);
+  BSE_OBJECT_SET_FLAGS (self, BSE_SUPER_FLAG_NEEDS_CONTEXT);
   self->midi_channel_id = 1;
   self->n_voices = 16;
-  self->volume_factor = bse_dB_to_factor (BSE_DFL_MASTER_VOLUME_dB);
+  self->volume_factor = bse_db_to_factor (0);
   
   /* midi voice modules */
   self->voice_input = bse_container_new_child (BSE_CONTAINER (self), BSE_TYPE_MIDI_VOICE_INPUT, NULL);
@@ -282,26 +214,27 @@ bse_midi_synth_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static BseItemSeq*
-bse_midi_synth_list_items (BseItem    *item,
-                           guint       param_id,
-                           GParamSpec *pspec)
+static void
+bse_midi_synth_get_candidates (BseItem               *item,
+                               guint                  param_id,
+                               BsePropertyCandidates *pc,
+                               GParamSpec            *pspec)
 {
   BseMidiSynth *self = BSE_MIDI_SYNTH (item);
-  BseItemSeq *iseq = bse_item_seq_new ();
   switch (param_id)
     {
     case PROP_SNET:
-      bse_item_gather_items_typed (item, iseq, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
+      bse_property_candidate_relabel (pc, _("Available Synthesizers"), _("List of available synthesis networks to choose a MIDI instrument from"));
+      bse_item_gather_items_typed (item, pc->items, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
       break;
-    case PROP_POST_NET:
-      bse_item_gather_items_typed (item, iseq, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
+    case PROP_PNET:
+      bse_property_candidate_relabel (pc, _("Available Postprocessors"), _("List of available synthesis networks to choose a postprocessor from"));
+      bse_item_gather_items_typed (item, pc->items, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
       break;
     }
-  return iseq;
 }
 
 static void
@@ -349,7 +282,7 @@ bse_midi_synth_set_property (GObject      *object,
                         NULL);
         }
       break;
-    case PROP_POST_NET:
+    case PROP_PNET:
       if (!BSE_SOURCE_PREPARED (self))
         {
           if (self->pnet)
@@ -390,7 +323,7 @@ bse_midi_synth_set_property (GObject      *object,
       g_object_notify (self, "volume_perc");
       break;
     case PROP_VOLUME_dB:
-      self->volume_factor = bse_dB_to_factor (sfi_value_get_real (value));
+      self->volume_factor = bse_db_to_factor (sfi_value_get_real (value));
       g_object_set (self->output, /* no undo */
                     "master_volume_f", self->volume_factor,
                     NULL);
@@ -423,7 +356,7 @@ bse_midi_synth_get_property (GObject    *object,
     case PROP_SNET:
       bse_value_set_object (value, self->snet);
       break;
-    case PROP_POST_NET:
+    case PROP_PNET:
       bse_value_set_object (value, self->pnet);
       break;
     case PROP_MIDI_CHANNEL:
@@ -436,7 +369,7 @@ bse_midi_synth_get_property (GObject    *object,
       sfi_value_set_real (value, self->volume_factor);
       break;
     case PROP_VOLUME_dB:
-      sfi_value_set_real (value, bse_dB_from_factor (self->volume_factor, BSE_MIN_VOLUME_dB));
+      sfi_value_set_real (value, bse_db_from_factor (self->volume_factor, BSE_MIN_VOLUME_dB));
       break;
     case PROP_VOLUME_PERC:
       sfi_value_set_int (value, self->volume_factor * 100.0 + 0.5);
@@ -450,7 +383,7 @@ bse_midi_synth_get_property (GObject    *object,
 static void
 bse_midi_synth_context_create (BseSource *source,
 			       guint      context_handle,
-			       GslTrans  *trans)
+			       BseTrans  *trans)
 {
   BseMidiSynth *self = BSE_MIDI_SYNTH (source);
   BseSNet *snet = BSE_SNET (self);
@@ -464,5 +397,89 @@ bse_midi_synth_context_create (BseSource *source,
       guint i;
       for (i = 0; i < self->n_voices; i++)
 	bse_snet_context_clone_branch (snet, context_handle, self->context_merger, mcontext, trans);
+
+      bse_midi_receiver_channel_enable_poly (mcontext.midi_receiver, mcontext.midi_channel);
     }
 }
+
+static void
+bse_midi_synth_context_dismiss (BseSource *source,
+                                guint      context_handle,
+                                BseTrans  *trans)
+{
+  BseMidiSynth *self = BSE_MIDI_SYNTH (source);
+  BseSNet *snet = BSE_SNET (self);
+
+  /* chain parent class' handler */
+  BSE_SOURCE_CLASS (parent_class)->context_dismiss (source, context_handle, trans);
+
+  if (!bse_snet_context_is_branch (snet, context_handle))
+    {
+      BseMidiContext mcontext = bse_snet_get_midi_context (snet, context_handle);
+      bse_midi_receiver_channel_disable_poly (mcontext.midi_receiver, mcontext.midi_channel);
+    }
+}
+
+static void
+bse_midi_synth_class_init (BseMidiSynthClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  BseItemClass *item_class = BSE_ITEM_CLASS (class);
+  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
+  
+  parent_class = g_type_class_peek_parent (class);
+  
+  gobject_class->set_property = bse_midi_synth_set_property;
+  gobject_class->get_property = bse_midi_synth_get_property;
+  gobject_class->finalize = bse_midi_synth_finalize;
+  
+  item_class->get_candidates = bse_midi_synth_get_candidates;
+  
+  source_class->context_create = bse_midi_synth_context_create;
+  source_class->context_dismiss = bse_midi_synth_context_dismiss;
+  
+  bse_object_class_add_param (object_class, _("MIDI Instrument"),
+			      PROP_MIDI_CHANNEL,
+			      sfi_pspec_int ("midi_channel", _("MIDI Channel"), NULL,
+					     1, 1, BSE_MIDI_MAX_CHANNELS, 1,
+					     SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale:skip-default:unprepared"));
+  bse_object_class_add_param (object_class, _("MIDI Instrument"),
+			      PROP_N_VOICES,
+			      sfi_pspec_int ("n_voices", _("Max Voices"), _("Maximum number of voices for simultaneous playback"),
+					     16, 1, 256, 1,
+					     SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale"));
+  bse_object_class_add_param (object_class, _("MIDI Instrument"),
+			      PROP_SNET,
+			      bse_param_spec_object ("snet", _("Synthesizer"), _("Synthesis network to be used as MIDI instrument"),
+						     BSE_TYPE_CSYNTH, SFI_PARAM_STANDARD ":unprepared"));
+  bse_object_class_add_param (object_class, _("MIDI Instrument"),
+                              PROP_PNET,
+                              bse_param_spec_object ("pnet", _("Postprocessor"), _("Synthesis network to be used as postprocessor"),
+                                                     BSE_TYPE_CSYNTH,
+                                                     SFI_PARAM_STANDARD ":unprepared"));
+  bse_object_class_add_param (object_class, _("Adjustments"),
+			      PROP_VOLUME_f,
+			      sfi_pspec_real ("volume_f", _("Master [float]"), NULL,
+					      bse_db_to_factor (0),
+					      0, bse_db_to_factor (BSE_MAX_VOLUME_dB), 0.1,
+					      SFI_PARAM_STORAGE));
+  bse_object_class_add_param (object_class, _("Adjustments"),
+			      PROP_VOLUME_dB,
+			      sfi_pspec_real ("volume_dB", _("Master [dB]"), NULL,
+					      0,
+					      BSE_MIN_VOLUME_dB, BSE_MAX_VOLUME_dB,
+					      BSE_GCONFIG (step_volume_dB),
+					      SFI_PARAM_GUI ":dial"));
+  bse_object_class_add_param (object_class, _("Adjustments"),
+			      PROP_VOLUME_PERC,
+			      sfi_pspec_int ("volume_perc", _("Master [%]"), NULL,
+					     bse_db_to_factor (0) * 100,
+					     0, bse_db_to_factor (BSE_MAX_VOLUME_dB) * 100, 1,
+					     SFI_PARAM_GUI ":dial"));
+  bse_object_class_add_param (object_class, _("Playback Settings"),
+			      PROP_AUTO_ACTIVATE,
+			      sfi_pspec_bool ("auto_activate", NULL, NULL,
+					      TRUE, /* change default */
+					      /* override parent property */ 0));
+}
