@@ -221,8 +221,8 @@ note_from_freq_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpe
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = sfi_pspec_real ("freq", "Frequency", NULL,
-			  BSE_KAMMER_FREQUENCY_f,
-			  0, BSE_MAX_FREQUENCY_f,
+			  BSE_KAMMER_FREQUENCY,
+			  0, BSE_MAX_FREQUENCY,
 			  10.0, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_boxed ("note-description", "Note Description", NULL,
                                 BSE_TYPE_NOTE_DESCRIPTION, SFI_PARAM_STANDARD);
