@@ -21,7 +21,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <sfi/gbsearcharray.h>
-#include <bse/gslengine.h>
+#include <bse/bseengine.h>
 #include <bse/gslcommon.h>
 #include "bsesnet.h"
 
@@ -52,18 +52,6 @@ enum
 
 
 /* --- prototypes --- */
-static void      bse_snet_class_init             (BseSNetClass   *class);
-static void      bse_snet_init                   (BseSNet        *snet);
-static void      bse_snet_dispose                (GObject        *object);
-static void      bse_snet_finalize               (GObject        *object);
-static void      bse_snet_set_property           (GObject	 *object,
-						  guint           param_id,
-						  const GValue   *value,
-						  GParamSpec     *pspec);
-static void      bse_snet_get_property           (GObject        *object,
-						  guint           param_id,
-						  GValue         *value,
-						  GParamSpec     *pspec);
 static void      bse_snet_add_item               (BseContainer   *container,
 						  BseItem        *item);
 static void      bse_snet_forall_items           (BseContainer   *container,
@@ -78,13 +66,13 @@ static gint	 snet_ports_compare              (gconstpointer   bsearch_node1, /*
 						  gconstpointer   bsearch_node2);
 static void      bse_snet_context_create	 (BseSource      *source,
 						  guint           context_handle,
-						  GslTrans       *trans);
+						  BseTrans       *trans);
 static void      bse_snet_context_connect	 (BseSource      *source,
 						  guint           context_handle,
-						  GslTrans       *trans);
+						  BseTrans       *trans);
 static void      bse_snet_context_dismiss	 (BseSource      *source,
 						  guint           context_handle,
-						  GslTrans       *trans);
+						  BseTrans       *trans);
 static GSList*	 snet_context_children		 (BseContainer	 *container);
 
 
@@ -99,60 +87,6 @@ static const GBSearchConfig port_array_config = {
 
 
 /* --- functions --- */
-BSE_BUILTIN_TYPE (BseSNet)
-{
-  static const GTypeInfo type_info = {
-    sizeof (BseSNetClass),
-    (GBaseInitFunc) NULL,
-    (GBaseFinalizeFunc) NULL,
-    (GClassInitFunc) bse_snet_class_init,
-    (GClassFinalizeFunc) NULL,
-    NULL /* class_data */,
-    sizeof (BseSNet),
-    0 /* n_preallocs */,
-    (GInstanceInitFunc) bse_snet_init,
-  };
-  g_assert (BSE_SNET_FLAGS_USHIFT < BSE_OBJECT_FLAGS_MAX_SHIFT);
-  return bse_type_register_abstract (BSE_TYPE_SUPER, "BseSNet", "BSE Synthesis (Filter) Network", &type_info);
-}
-
-static void
-bse_snet_class_init (BseSNetClass *class)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
-  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
-  BseContainerClass *container_class = BSE_CONTAINER_CLASS (class);
-  
-  parent_class = g_type_class_peek_parent (class);
-  
-  gobject_class->set_property = bse_snet_set_property;
-  gobject_class->get_property = bse_snet_get_property;
-  gobject_class->dispose = bse_snet_dispose;
-  gobject_class->finalize = bse_snet_finalize;
-  
-  source_class->prepare = bse_snet_prepare;
-  source_class->context_create = bse_snet_context_create;
-  source_class->context_connect = bse_snet_context_connect;
-  source_class->context_dismiss = bse_snet_context_dismiss;
-  source_class->reset = bse_snet_reset;
-  
-  container_class->add_item = bse_snet_add_item;
-  container_class->remove_item = bse_snet_remove_item;
-  container_class->forall_items = bse_snet_forall_items;
-  container_class->context_children = snet_context_children;
-  container_class->release_children = bse_snet_release_children;
-  
-  bse_object_class_add_param (object_class, "Playback Settings",
-			      PARAM_AUTO_ACTIVATE,
-			      sfi_pspec_bool ("auto_activate", "Auto Activate",
-					      "Automatic activation only needs to be enabled for synthesis networks "
-					      "that don't use virtual ports for their input and output",
-					      FALSE, SFI_PARAM_STANDARD));
-  signal_port_unregistered = bse_object_class_add_signal (object_class, "port_unregistered",
-							  G_TYPE_NONE, 0);
-}
-
 static void
 bse_snet_init (BseSNet *snet)
 {
@@ -197,10 +131,6 @@ bse_snet_release_children (BseContainer *container)
 
   while (snet->sources)
     bse_container_remove_item (container, sfi_ring_pop_head (&snet->sources));
-  if (snet->iport_names)
-    g_warning ("%s: leaking %cport \"%s\"", G_STRLOC, 'i', (gchar*) snet->iport_names->data);
-  if (snet->oport_names)
-    g_warning ("%s: leaking %cport \"%s\"", G_STRLOC, 'o', (gchar*) snet->oport_names->data);
   
   /* chain parent class' handler */
   BSE_CONTAINER_CLASS (parent_class)->release_children (container);
@@ -209,8 +139,6 @@ bse_snet_release_children (BseContainer *container)
 static void
 bse_snet_dispose (GObject *object)
 {
-  // BseSNet *snet = BSE_SNET (object);
-  
   /* chain parent class' handler */
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -219,12 +147,21 @@ static void
 bse_snet_finalize (GObject *object)
 {
   BseSNet *snet = BSE_SNET (object);
+  BseContainer *container = BSE_CONTAINER (object);
   
+  while (snet->sources)
+    bse_container_remove_item (container, sfi_ring_pop_head (&snet->sources));
+  while (snet->isources)
+    bse_container_remove_item (container, sfi_ring_pop_head (&snet->isources));
   if (snet->port_unregistered_id)
     {
       bse_idle_remove (snet->port_unregistered_id);
       snet->port_unregistered_id = 0;
     }
+  if (snet->iport_names)
+    g_warning ("%s: %s: leaking %cport \"%s\"", G_STRLOC, G_OBJECT_TYPE_NAME (snet), 'i', (gchar*) snet->iport_names->data);
+  if (snet->oport_names)
+    g_warning ("%s: %s: leaking %cport \"%s\"", G_STRLOC, G_OBJECT_TYPE_NAME (snet), 'o', (gchar*) snet->oport_names->data);
   
   /* chain parent class' handler */
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -264,9 +201,9 @@ bse_snet_set_property (GObject      *object,
     {
     case PARAM_AUTO_ACTIVATE:
       if (sfi_value_get_bool (value))
-	BSE_OBJECT_SET_FLAGS (self, BSE_SUPER_FLAG_NEEDS_SEQUENCER_CONTEXT);
+	BSE_OBJECT_SET_FLAGS (self, BSE_SUPER_FLAG_NEEDS_CONTEXT);
       else
-	BSE_OBJECT_UNSET_FLAGS (self, BSE_SUPER_FLAG_NEEDS_SEQUENCER_CONTEXT);
+	BSE_OBJECT_UNSET_FLAGS (self, BSE_SUPER_FLAG_NEEDS_CONTEXT);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
@@ -285,7 +222,7 @@ bse_snet_get_property (GObject    *object,
   switch (param_id)
     {
     case PARAM_AUTO_ACTIVATE:
-      sfi_value_set_bool (value, BSE_SUPER_NEEDS_SEQUENCER_CONTEXT (self));
+      sfi_value_set_bool (value, BSE_SUPER_NEEDS_CONTEXT (self));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
@@ -559,9 +496,9 @@ void
 bse_snet_set_iport_src (BseSNet     *snet,
 			const gchar *name,
 			guint        snet_context,
-			GslModule   *omodule,
+			BseModule   *omodule,
 			guint        ostream,
-			GslTrans    *trans)
+			BseTrans    *trans)
 {
   BseSNetPort *port;
   
@@ -569,7 +506,7 @@ bse_snet_set_iport_src (BseSNet     *snet,
   g_return_if_fail (name != NULL);
   g_return_if_fail (bse_source_has_context (BSE_SOURCE (snet), snet_context));
   if (omodule)
-    g_return_if_fail (ostream < GSL_MODULE_N_OSTREAMS (omodule));
+    g_return_if_fail (ostream < BSE_MODULE_N_OSTREAMS (omodule));
   g_return_if_fail (trans != NULL);
   
   port = port_lookup (snet, name, snet_context, TRUE);
@@ -581,11 +518,11 @@ bse_snet_set_iport_src (BseSNet     *snet,
     ostream = G_MAXUINT;
   
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_disconnect (port->dest_imodule, port->dest_istream));
+    bse_trans_add (trans, bse_job_disconnect (port->dest_imodule, port->dest_istream));
   port->src_omodule = omodule;
   port->src_ostream = ostream;
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_connect (port->src_omodule, port->src_ostream,
+    bse_trans_add (trans, bse_job_connect (port->src_omodule, port->src_ostream,
 					   port->dest_imodule, port->dest_istream));
   if (!port->dest_imodule && !port->src_omodule)
     port_delete (snet, port);
@@ -595,9 +532,9 @@ void
 bse_snet_set_iport_dest (BseSNet     *snet,
 			 const gchar *name,
 			 guint        snet_context,
-			 GslModule   *imodule,
+			 BseModule   *imodule,
 			 guint        istream,
-			 GslTrans    *trans)
+			 BseTrans    *trans)
 {
   BseSNetPort *port;
   
@@ -605,7 +542,7 @@ bse_snet_set_iport_dest (BseSNet     *snet,
   g_return_if_fail (name != NULL);
   g_return_if_fail (bse_source_has_context (BSE_SOURCE (snet), snet_context));
   if (imodule)
-    g_return_if_fail (istream < GSL_MODULE_N_ISTREAMS (imodule));
+    g_return_if_fail (istream < BSE_MODULE_N_ISTREAMS (imodule));
   g_return_if_fail (trans != NULL);
   
   port = port_lookup (snet, name, snet_context, TRUE);
@@ -617,11 +554,11 @@ bse_snet_set_iport_dest (BseSNet     *snet,
     istream = G_MAXUINT;
   
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_disconnect (port->dest_imodule, port->dest_istream));
+    bse_trans_add (trans, bse_job_disconnect (port->dest_imodule, port->dest_istream));
   port->dest_imodule = imodule;
   port->dest_istream = istream;
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_connect (port->src_omodule, port->src_ostream,
+    bse_trans_add (trans, bse_job_connect (port->src_omodule, port->src_ostream,
 					   port->dest_imodule, port->dest_istream));
   if (!port->dest_imodule && !port->src_omodule)
     port_delete (snet, port);
@@ -631,9 +568,9 @@ void
 bse_snet_set_oport_src (BseSNet     *snet,
 			const gchar *name,
 			guint        snet_context,
-			GslModule   *omodule,
+			BseModule   *omodule,
 			guint        ostream,
-			GslTrans    *trans)
+			BseTrans    *trans)
 {
   BseSNetPort *port;
   
@@ -641,7 +578,7 @@ bse_snet_set_oport_src (BseSNet     *snet,
   g_return_if_fail (name != NULL);
   g_return_if_fail (bse_source_has_context (BSE_SOURCE (snet), snet_context));
   if (omodule)
-    g_return_if_fail (ostream < GSL_MODULE_N_OSTREAMS (omodule));
+    g_return_if_fail (ostream < BSE_MODULE_N_OSTREAMS (omodule));
   g_return_if_fail (trans != NULL);
   
   port = port_lookup (snet, name, snet_context, FALSE);
@@ -653,11 +590,11 @@ bse_snet_set_oport_src (BseSNet     *snet,
     ostream = G_MAXUINT;
   
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_disconnect (port->dest_imodule, port->dest_istream));
+    bse_trans_add (trans, bse_job_disconnect (port->dest_imodule, port->dest_istream));
   port->src_omodule = omodule;
   port->src_ostream = ostream;
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_connect (port->src_omodule, port->src_ostream,
+    bse_trans_add (trans, bse_job_connect (port->src_omodule, port->src_ostream,
 					   port->dest_imodule, port->dest_istream));
   if (!port->dest_imodule && !port->src_omodule)
     port_delete (snet, port);
@@ -667,9 +604,9 @@ void
 bse_snet_set_oport_dest (BseSNet     *snet,
 			 const gchar *name,
 			 guint        snet_context,
-			 GslModule   *imodule,
+			 BseModule   *imodule,
 			 guint        istream,
-			 GslTrans    *trans)
+			 BseTrans    *trans)
 {
   BseSNetPort *port;
   
@@ -677,7 +614,7 @@ bse_snet_set_oport_dest (BseSNet     *snet,
   g_return_if_fail (name != NULL);
   g_return_if_fail (bse_source_has_context (BSE_SOURCE (snet), snet_context));
   if (imodule)
-    g_return_if_fail (istream < GSL_MODULE_N_ISTREAMS (imodule));
+    g_return_if_fail (istream < BSE_MODULE_N_ISTREAMS (imodule));
   g_return_if_fail (trans != NULL);
   
   port = port_lookup (snet, name, snet_context, FALSE);
@@ -689,11 +626,11 @@ bse_snet_set_oport_dest (BseSNet     *snet,
     istream = G_MAXUINT;
   
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_disconnect (port->dest_imodule, port->dest_istream));
+    bse_trans_add (trans, bse_job_disconnect (port->dest_imodule, port->dest_istream));
   port->dest_imodule = imodule;
   port->dest_istream = istream;
   if (port->src_omodule && port->dest_imodule)
-    gsl_trans_add (trans, gsl_job_connect (port->src_omodule, port->src_ostream,
+    bse_trans_add (trans, bse_job_connect (port->src_omodule, port->src_ostream,
 					   port->dest_imodule, port->dest_istream));
   if (!port->dest_imodule && !port->src_omodule)
     port_delete (snet, port);
@@ -740,7 +677,7 @@ create_context_data (BseSNet         *self,
 static void
 free_context_data (BseSource *source,
 		   gpointer   data,
-		   GslTrans  *trans)
+		   BseTrans  *trans)
 {
   BseSNet *self = BSE_SNET (source);
   ContextData *cdata = data;
@@ -772,7 +709,7 @@ free_context_data (BseSource *source,
 guint
 bse_snet_create_context (BseSNet         *self,
                          BseMidiContext   mcontext,
-			 GslTrans        *trans)
+			 BseTrans        *trans)
 {
   ContextData *cdata;
   guint cid;
@@ -796,7 +733,7 @@ bse_snet_context_clone_branch (BseSNet         *self,
 			       guint            context,
 			       BseSource       *context_merger,
                                BseMidiContext   mcontext,
-			       GslTrans        *trans)
+			       BseTrans        *trans)
 {
   SfiRing *ring;
   guint bcid = 0;
@@ -926,7 +863,7 @@ bse_snet_reset (BseSource *source)
 static void
 bse_snet_context_create (BseSource *source,
 			 guint      context_handle,
-			 GslTrans  *trans)
+			 BseTrans  *trans)
 {
   BseSNet *self = BSE_SNET (source);
   
@@ -952,7 +889,7 @@ bse_snet_context_create (BseSource *source,
 static void
 bse_snet_context_connect (BseSource *source,
 			  guint      context_handle,
-			  GslTrans  *trans)
+			  BseTrans  *trans)
 {
   BseSNet *self = BSE_SNET (source);
   ContextData *cdata = find_context_data (self, context_handle);
@@ -968,7 +905,7 @@ bse_snet_context_connect (BseSource *source,
 static void
 bse_snet_context_dismiss (BseSource *source,
 			  guint      context_handle,
-			  GslTrans  *trans)
+			  BseTrans  *trans)
 {
   BseSNet *self = BSE_SNET (source);
   ContextData *cdata = find_context_data (self, context_handle);
@@ -979,3 +916,57 @@ bse_snet_context_dismiss (BseSource *source,
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->context_dismiss (source, context_handle, trans);
 }
+
+static void
+bse_snet_class_init (BseSNetClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
+  BseContainerClass *container_class = BSE_CONTAINER_CLASS (class);
+  
+  parent_class = g_type_class_peek_parent (class);
+  
+  gobject_class->set_property = bse_snet_set_property;
+  gobject_class->get_property = bse_snet_get_property;
+  gobject_class->dispose = bse_snet_dispose;
+  gobject_class->finalize = bse_snet_finalize;
+  
+  source_class->prepare = bse_snet_prepare;
+  source_class->context_create = bse_snet_context_create;
+  source_class->context_connect = bse_snet_context_connect;
+  source_class->context_dismiss = bse_snet_context_dismiss;
+  source_class->reset = bse_snet_reset;
+  
+  container_class->add_item = bse_snet_add_item;
+  container_class->remove_item = bse_snet_remove_item;
+  container_class->forall_items = bse_snet_forall_items;
+  container_class->context_children = snet_context_children;
+  container_class->release_children = bse_snet_release_children;
+  
+  bse_object_class_add_param (object_class, "Playback Settings",
+			      PARAM_AUTO_ACTIVATE,
+			      sfi_pspec_bool ("auto_activate", "Auto Activate",
+					      "Automatic activation only needs to be enabled for synthesis networks "
+					      "that don't use virtual ports for their input and output",
+					      FALSE, SFI_PARAM_STANDARD ":skip-default"));
+  signal_port_unregistered = bse_object_class_add_signal (object_class, "port_unregistered",
+							  G_TYPE_NONE, 0);
+}
+
+BSE_BUILTIN_TYPE (BseSNet)
+{
+  static const GTypeInfo type_info = {
+    sizeof (BseSNetClass),
+    (GBaseInitFunc) NULL,
+    (GBaseFinalizeFunc) NULL,
+    (GClassInitFunc) bse_snet_class_init,
+    (GClassFinalizeFunc) NULL,
+    NULL /* class_data */,
+    sizeof (BseSNet),
+    0 /* n_preallocs */,
+    (GInstanceInitFunc) bse_snet_init,
+  };
+  g_assert (BSE_SNET_FLAGS_USHIFT < BSE_OBJECT_FLAGS_MAX_SHIFT);
+  return bse_type_register_abstract (BSE_TYPE_SUPER, "BseSNet", "BSE Synthesis (Filter) Network", &type_info);
+}
