@@ -18,7 +18,9 @@
 #include "bsestorage.h"
 #include "bseitem.h"
 #include "gsldatahandle.h"
+#include "gsldatahandle-vorbis.h"
 #include "gsldatautils.h"
+#include "gslcommon.h"
 #include "bseproject.h"
 #include "bseparasite.h"
 #include <unistd.h>
@@ -39,7 +41,8 @@ struct _BseStorageDBlock
 {
   gulong         id;
   GslDataHandle *dhandle;
-  guint          n_channels;
+  guint          n_channels : 16;
+  guint          needs_close : 1;
   gfloat         mix_freq;
   gfloat         osc_freq;
 };
@@ -80,6 +83,7 @@ static GTokenType compat_parse_data_handle         (BseStorage       *self,
 /* --- variables --- */
 static gpointer parent_class = NULL;
 static GQuark   quark_raw_data_handle = 0;
+static GQuark   quark_vorbis_data_handle = 0;
 static GQuark   quark_dblock_data_handle = 0;
 static GQuark   quark_bse_storage_binary_v0 = 0;
 
@@ -114,6 +118,7 @@ bse_storage_class_init (BseStorageClass *class)
   parent_class = g_type_class_peek_parent (class);
 
   quark_raw_data_handle = g_quark_from_static_string ("raw-data-handle");
+  quark_vorbis_data_handle = g_quark_from_static_string ("vorbis-data-handle");
   quark_dblock_data_handle = g_quark_from_static_string ("dblock-data-handle");
   quark_bse_storage_binary_v0 = g_quark_from_static_string ("BseStorageBinaryV0");
 
@@ -129,7 +134,9 @@ bse_storage_init (BseStorage *self)
   self->referenced_items = NULL;
   /* reading */
   self->rstore = NULL;
+  self->path_table = NULL;
   self->item_links = NULL;
+  self->restorable_objects = NULL;
   /* misc */
   self->dblocks = NULL;
   self->n_dblocks = 0;
@@ -190,11 +197,14 @@ bse_storage_reset (BseStorage *self)
 
   if (self->rstore)
     {
-      bse_storage_resolve_item_links (self);
+      bse_storage_finish_parsing (self);
       g_hash_table_destroy (self->path_table);
       self->path_table = NULL;
       sfi_rstore_destroy (self->rstore);
       self->rstore = NULL;
+      if (self->restorable_objects)
+        sfi_ppool_destroy (self->restorable_objects);
+      self->restorable_objects = NULL;
     }
 
   if (self->wstore)
@@ -214,6 +224,8 @@ bse_storage_reset (BseStorage *self)
   for (i = 0; i < self->n_dblocks; i++)
     {
       bse_id_free (self->dblocks[i].id);
+      if (self->dblocks[i].needs_close)
+        gsl_data_handle_close (self->dblocks[i].dhandle);
       gsl_data_handle_unref (self->dblocks[i].dhandle);
     }
   g_free (self->dblocks);
@@ -234,6 +246,14 @@ bse_storage_add_dblock (BseStorage    *self,
   self->dblocks = g_renew (BseStorageDBlock, self->dblocks, self->n_dblocks);
   self->dblocks[i].id = bse_id_alloc ();
   self->dblocks[i].dhandle = gsl_data_handle_ref (dhandle);
+  if (GSL_DATA_HANDLE_OPENED (dhandle))
+    {
+      /* keep data handles opened to protect against rewrites */
+      gsl_data_handle_open (dhandle);
+      self->dblocks[i].needs_close = TRUE;
+    }
+  else
+    self->dblocks[i].needs_close = FALSE;
   self->dblocks[i].n_channels = gsl_data_handle_n_channels (dhandle);
   self->dblocks[i].mix_freq = gsl_data_handle_mix_freq (dhandle);
   self->dblocks[i].osc_freq = gsl_data_handle_osc_freq (dhandle);
@@ -284,6 +304,7 @@ bse_storage_input_text (BseStorage  *self,
   self->rstore->parser_this = self;
   sfi_rstore_input_text (self->rstore, text, text_name);
   self->path_table = g_hash_table_new_full (uname_child_hash, uname_child_equals, NULL, uname_child_free);
+  self->restorable_objects = sfi_ppool_new ();
 }
 
 BseErrorType
@@ -299,6 +320,7 @@ bse_storage_input_file (BseStorage  *self,
     return bse_error_from_errno (errno, BSE_ERROR_FILE_OPEN_FAILED);
   self->rstore->parser_this = self;
   self->path_table = g_hash_table_new_full (uname_child_hash, uname_child_equals, NULL, uname_child_free);
+  self->restorable_objects = sfi_ppool_new ();
 
   return BSE_ERROR_NONE;
 }
@@ -369,7 +391,30 @@ storage_add_item_link (BseStorage           *self,
 }
 
 void
-bse_storage_resolve_item_links (BseStorage *self)
+bse_storage_add_restorable (BseStorage             *self,
+                            BseObject              *object)
+{
+  g_return_if_fail (BSE_IS_STORAGE (self));
+  g_return_if_fail (self->rstore);
+  g_return_if_fail (self->restorable_objects);
+  g_return_if_fail (BSE_IS_OBJECT (object));
+  g_return_if_fail (BSE_OBJECT_IN_RESTORE (object));
+
+  sfi_ppool_set (self->restorable_objects, object);
+}
+
+static gboolean
+storage_restorable_objects_foreach (gpointer        data,
+                                    gpointer        pointer)
+{
+  // BseStorage *self = BSE_STORAGE (data);
+  BseObject *object = BSE_OBJECT (pointer);
+  bse_object_restore_finish (object);
+  return TRUE;
+}
+
+void
+bse_storage_finish_parsing (BseStorage *self)
 {
   g_return_if_fail (BSE_IS_STORAGE (self));
   g_return_if_fail (self->rstore != NULL);
@@ -431,6 +476,12 @@ bse_storage_resolve_item_links (BseStorage *self)
       g_free (ilink->upath);
       g_free (ilink);
     }
+
+  /* finish restorables */
+  sfi_ppool_foreach (self->restorable_objects, storage_restorable_objects_foreach, self);
+  /* clear pool */
+  sfi_ppool_destroy (self->restorable_objects);
+  self->restorable_objects = sfi_ppool_new();
 }
 
 const gchar*
@@ -621,8 +672,12 @@ restore_item_property (BseItem    *item,
     }
 
   /* set property value while preserving the object uname */
-  g_object_set_property (G_OBJECT (item), /* no undo */
-                         pspec->name, &value);
+  if ((pspec->flags & G_PARAM_WRITABLE) && !(pspec->flags & G_PARAM_CONSTRUCT_ONLY))
+    g_object_set_property (G_OBJECT (item), /* no undo */
+                           pspec->name, &value);
+  else
+    bse_storage_warn (self, "ignoring non-writable object property \"%s\" of type `%s'",
+                      pspec->name, g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
   g_value_unset (&value);
 
   return G_TOKEN_NONE;
@@ -722,7 +777,7 @@ restore_container_child (BseContainer *container,
   bse_item_compat_setup (item, self->major_version, self->minor_version, self->micro_version);
 
   storage_path_table_insert (self, container, uname, item);
-
+  
   /* restore_item reads out closing parenthesis */
   g_object_ref (item);
   expected_token = bse_storage_parse_rest (self, item, item_restore_try_statement, NULL);
@@ -753,6 +808,8 @@ item_restore_try_statement (gpointer    item,
    * anything else       - encountered (syntax/semantic) error during parsing
    */
 
+  bse_object_restore_start (BSE_OBJECT (item), self);
+
   if (expected_token == SFI_TOKEN_UNMATCHED)
     expected_token = restore_item_property (item, self);
 
@@ -938,6 +995,7 @@ bse_storage_parse_item_link (BseStorage           *self,
 
   g_scanner_get_next_token (scanner);
 
+  bse_object_restore_start (BSE_OBJECT (from_item), self);
   if (sfi_serial_check_parse_null_token (scanner))
     {
       ilink = storage_add_item_link (self, from_item, restore_link, data, NULL);
@@ -1168,28 +1226,20 @@ void
 bse_storage_putf (BseStorage *self,
                   gfloat      vfloat)
 {
-  gchar numbuf[G_ASCII_DTOSTR_BUF_SIZE + 1] = "";
-
   g_return_if_fail (BSE_IS_STORAGE (self));
   g_return_if_fail (self->wstore);
 
-  g_ascii_formatd (numbuf, G_ASCII_DTOSTR_BUF_SIZE, "%.7g", vfloat);
-
-  bse_storage_puts (self, numbuf);
+  sfi_wstore_putf (self->wstore, vfloat);
 }
 
 void
 bse_storage_putd (BseStorage *self,
                   gdouble     vdouble)
 {
-  gchar numbuf[G_ASCII_DTOSTR_BUF_SIZE + 1] = "";
-
   g_return_if_fail (BSE_IS_STORAGE (self));
   g_return_if_fail (self->wstore);
 
-  g_ascii_formatd (numbuf, G_ASCII_DTOSTR_BUF_SIZE, "%.17g", vdouble);
-
-  bse_storage_puts (self, numbuf);
+  sfi_wstore_putd (self->wstore, vdouble);
 }
 
 void
@@ -1217,7 +1267,11 @@ bse_storage_printf (BseStorage  *self,
   g_return_if_fail (BSE_IS_STORAGE (self));
   g_return_if_fail (self->wstore);
   g_return_if_fail (format != NULL);
-  
+
+  const gchar *ldir = g_printf_find_localised_directive (format);
+  if (ldir)
+    g_warning ("%s: encountered localised directive \"%s\" in format string: \"%s\"", G_STRFUNC, ldir, format);
+
   va_start (args, format);
   buffer = g_strdup_vprintf (format, args);
   va_end (args);
@@ -1227,15 +1281,67 @@ bse_storage_printf (BseStorage  *self,
   g_free (buffer);
 }
 
-static void
-put_dblock_data_handle (BseStorage    *self,
-                        guint          significant_bits,
-                        GslDataHandle *dhandle)
+void
+bse_storage_put_xinfos (BseStorage *self,
+                        gchar     **xinfos)
 {
-  gulong id = bse_storage_add_dblock (self, dhandle);
+  xinfos = bse_xinfos_dup_consolidated (xinfos, FALSE);
+  if (xinfos && xinfos[0])
+    {
+      bse_storage_break (self);
+      gchar *str = g_strescape (xinfos[0], NULL);
+      bse_storage_printf (self, " (\"%s\"", str);
+      g_free (str);
+      guint i;
+      bse_storage_push_level (self);
+      for (i = 1; xinfos[i]; i++)
+        {
+          bse_storage_break (self);
+          str = g_strescape (xinfos[i], NULL);
+          bse_storage_printf (self, "\"%s\"", str);
+          g_free (str);
+        }
+      bse_storage_pop_level (self);
+      bse_storage_puts (self, ")");
+    }
+  else
+    bse_storage_printf (self, "#f");
+  g_strfreev (xinfos);
+}
 
-  bse_storage_break (self);
-  bse_storage_printf (self, "(%s %lu)", g_quark_to_string (quark_dblock_data_handle), id);
+GTokenType
+bse_storage_parse_xinfos (BseStorage *self,
+                          gchar    ***xinfosp)
+{
+  GScanner *scanner = bse_storage_get_scanner (self);
+  g_scanner_get_next_token (scanner);
+  if (scanner->token == '#')    /* parse "#f" => NULL */
+    {
+      g_scanner_get_next_token (scanner);
+      if (scanner->token == 'f' || scanner->token == 'F')
+        {
+          *xinfosp = NULL;
+          return G_TOKEN_NONE;
+        }
+      /* everything else, even #t is bogus */
+      return 'f';
+    }
+  else if (scanner->token == '(')
+    {
+      gchar **xinfos = NULL;
+      while (g_scanner_get_next_token (scanner) != ')')
+        {
+          if (scanner->token == G_TOKEN_STRING)
+            xinfos = bse_xinfos_parse_assignment (xinfos, scanner->value.v_string);
+          else
+            return G_TOKEN_STRING;
+        }
+      *xinfosp = bse_xinfos_dup_consolidated (xinfos, FALSE);
+      g_strfreev (xinfos);
+      return G_TOKEN_NONE;
+    }
+  else
+    return '(';
 }
 
 static GTokenType
@@ -1277,6 +1383,7 @@ typedef struct {
   guint          opened : 1;
   guint          bpv, format, byte_order;
   BseStorage    *storage;
+  guint          length;
 } WStoreDHandle;
 
 static void
@@ -1291,7 +1398,6 @@ wstore_data_handle_destroy (gpointer data)
 
 static gint /* -errno || length */
 wstore_data_handle_reader (gpointer data,
-                           SfiNum   pos,
                            void    *buffer,
                            guint    blength)
 {
@@ -1300,30 +1406,28 @@ wstore_data_handle_reader (gpointer data,
   
   if (!wh->opened)
     {
-      GslErrorType error = gsl_data_handle_open (wh->dhandle);
+      BseErrorType error = gsl_data_handle_open (wh->dhandle);
       if (error)
         {
-          bse_storage_error (wh->storage, "failed to open data handle: %s", gsl_strerror (error));
+          bse_storage_error (wh->storage, "failed to open data handle: %s", bse_error_blurb (error));
           return -ENOENT;
         }
       wh->opened = TRUE;
     }
 
-  /* shouldn't need to seek, check alignment */
-  g_return_val_if_fail (pos % wh->bpv == 0, -EIO);
-
   /* catch end */
-  if (pos / wh->bpv >= gsl_data_handle_length (wh->dhandle))
+  if (wh->length >= gsl_data_handle_length (wh->dhandle))
     return 0;
 
   do
-    n = gsl_data_handle_read (wh->dhandle, pos / wh->bpv, blength / sizeof (gfloat), buffer);
+    n = gsl_data_handle_read (wh->dhandle, wh->length, blength / sizeof (gfloat), buffer);
   while (n < 0 && errno == EINTR);
   if (n < 0)    /* bail out */
     {
       bse_storage_error (wh->storage, "failed to read from data handle");
       return -EIO;
     }
+  wh->length += n;
 
   return gsl_conv_from_float_clip (wh->format, wh->byte_order, buffer, buffer, n);
 }
@@ -1333,9 +1437,6 @@ bse_storage_put_data_handle (BseStorage    *self,
                              guint          significant_bits,
                              GslDataHandle *dhandle)
 {
-  WStoreDHandle *wh;
-  guint format;
-
   g_return_if_fail (BSE_IS_STORAGE (self));
   g_return_if_fail (self->wstore);
   g_return_if_fail (dhandle != NULL);
@@ -1343,41 +1444,72 @@ bse_storage_put_data_handle (BseStorage    *self,
 
   if (BSE_STORAGE_DBLOCK_CONTAINED (self))
     {
-      put_dblock_data_handle (self, significant_bits, dhandle);
+      /* stored as binary data block in memory for undo storage */
+      gulong id = bse_storage_add_dblock (self, dhandle);
+      bse_storage_break (self);
+      bse_storage_printf (self, "(%s %lu)", g_quark_to_string (quark_dblock_data_handle), id);
       return;
     }
+  
+  GslDataHandle *test_handle, *tmp_handle = dhandle;
+  do    /* skip comment or cache handles */
+    {
+      test_handle = tmp_handle;
+      tmp_handle = gsl_data_handle_get_source (test_handle);
+    }
+  while (tmp_handle);   /* skip comment or cache handles */
+  GslVorbis1Handle *vhandle = gsl_vorbis1_handle_new (test_handle, gsl_vorbis_make_serialno());
+  if (vhandle)  /* save already compressed Ogg/Vorbis data */
+    {
+      bse_storage_break (self);
+      bse_storage_printf (self,
+                          "(%s %.7g",
+                          g_quark_to_string (quark_vorbis_data_handle),
+                          gsl_data_handle_osc_freq (dhandle));
+      bse_storage_push_level (self);
+      bse_storage_break (self);
+      gsl_vorbis1_handle_put_wstore (vhandle, self->wstore);
+      bse_storage_pop_level (self);
+      bse_storage_putc (self, ')');
+    }
+  else          /* save raw data handle */
+    {
+      if (significant_bits < 1)
+        significant_bits = 32;
+      guint format = gsl_data_handle_bit_depth (dhandle);
+      significant_bits = MIN (format, significant_bits);
+      if (significant_bits > 16)
+        format = GSL_WAVE_FORMAT_FLOAT;
+      else if (significant_bits <= 8)
+        format = GSL_WAVE_FORMAT_SIGNED_8;
+      else
+        format = GSL_WAVE_FORMAT_SIGNED_16;
+      
+      bse_storage_break (self);
+      bse_storage_printf (self,
+                          "(%s %u %s %s",
+                          g_quark_to_string (quark_raw_data_handle),
+                          gsl_data_handle_n_channels (dhandle),
+                          gsl_wave_format_to_string (format),
+                          gsl_byte_order_to_string (G_LITTLE_ENDIAN));
+      bse_storage_puts (self, " ");
+      bse_storage_putf (self, gsl_data_handle_mix_freq (dhandle));
+      bse_storage_puts (self, " ");
+      bse_storage_putf (self, gsl_data_handle_osc_freq (dhandle));
+      bse_storage_push_level (self);
+      bse_storage_break (self);
 
-  if (significant_bits < 1)
-    significant_bits = 32;
-  format = gsl_data_handle_bit_depth (dhandle);
-  significant_bits = MIN (format, significant_bits);
-  if (significant_bits > 16)
-    format = GSL_WAVE_FORMAT_FLOAT;
-  else if (significant_bits <= 8)
-    format = GSL_WAVE_FORMAT_SIGNED_8;
-  else
-    format = GSL_WAVE_FORMAT_SIGNED_16;
+      WStoreDHandle *wh = g_new0 (WStoreDHandle, 1);
+      wh->dhandle = gsl_data_handle_ref (dhandle);
+      wh->format = format;
+      wh->byte_order = G_LITTLE_ENDIAN;
+      wh->bpv = gsl_wave_format_byte_width (format);
+      wh->storage = self;
+      sfi_wstore_put_binary (self->wstore, wstore_data_handle_reader, wh, wstore_data_handle_destroy);
+      bse_storage_pop_level (self);
 
-  bse_storage_break (self);
-  bse_storage_printf (self,
-                      "(%s %u %s %s %.7g %.7g",
-                      g_quark_to_string (quark_raw_data_handle),
-                      gsl_data_handle_n_channels (dhandle),
-                      gsl_wave_format_to_string (format),
-                      gsl_byte_order_to_string (G_LITTLE_ENDIAN),
-                      gsl_data_handle_mix_freq (dhandle),
-                      gsl_data_handle_osc_freq (dhandle));
-  bse_storage_push_level (self);
-  bse_storage_break (self);
-  wh = g_new0 (WStoreDHandle, 1);
-  wh->dhandle = gsl_data_handle_ref (dhandle);
-  wh->format = format;
-  wh->byte_order = G_LITTLE_ENDIAN;
-  wh->bpv = gsl_wave_format_byte_width (format);
-  wh->storage = self;
-  sfi_wstore_put_binary (self->wstore, wstore_data_handle_reader, wh, wstore_data_handle_destroy);
-  bse_storage_pop_level (self);
-  bse_storage_putc (self, ')');
+      bse_storage_putc (self, ')');
+    }
 }
 
 static GTokenType
@@ -1443,7 +1575,7 @@ parse_raw_data_handle (BseStorage     *self,
     *data_handle_p = gsl_wave_handle_new (self->rstore->fname,
                                           n_channels, format, byte_order,
                                           mix_freq, osc_freq,
-                                          offset, length);
+                                          offset, length, NULL);
   if (n_channels_p)
     *n_channels_p = n_channels;
   if (mix_freq_p)
@@ -1453,13 +1585,63 @@ parse_raw_data_handle (BseStorage     *self,
   return G_TOKEN_NONE;
 }
 
+static GTokenType
+parse_vorbis_data_handle (BseStorage     *self,
+                          GslDataHandle **data_handle_p,
+                          guint          *n_channels_p,
+                          gfloat         *mix_freq_p,
+                          gfloat         *osc_freq_p)
+{
+  GScanner *scanner = bse_storage_get_scanner (self);
+  GTokenType token;
+
+  gfloat osc_freq;
+  g_scanner_get_next_token (scanner);
+  if (scanner->token == G_TOKEN_INT)
+    osc_freq = scanner->value.v_int64;
+  else if (scanner->token == G_TOKEN_FLOAT)
+    osc_freq = scanner->value.v_float;
+  else
+    return G_TOKEN_FLOAT;
+  if (osc_freq <= 0)
+    return bse_storage_warn_skip (self, "invalid oscillating frequency: %.7g", osc_freq);
+  if (osc_freq_p)
+    *osc_freq_p = osc_freq;
+
+  SfiNum offset, length;
+  token = sfi_rstore_parse_zbinary (self->rstore, &offset, &length);
+  if (token != G_TOKEN_NONE)
+    return token;
+
+  parse_or_return (scanner, ')');
+
+  if (length < 1)
+    {
+      bse_storage_warn (self, "encountered empty data handle");
+      *data_handle_p = NULL;
+    }
+  else
+    {
+      gfloat mix_freq;
+      *data_handle_p = gsl_data_handle_new_ogg_vorbis_zoffset (self->rstore->fname, osc_freq,
+                                                               offset, length, n_channels_p, &mix_freq);
+      if (osc_freq <= 0 || mix_freq < 4000 || osc_freq >= mix_freq / 2)
+        return bse_storage_warn_skip (self, "invalid oscillating/mixing frequencies: %.7g/%.7g", osc_freq, mix_freq);
+      if (mix_freq_p)
+        *mix_freq_p = mix_freq;
+    }
+  return G_TOKEN_NONE;
+}
+
 gboolean
 bse_storage_match_data_handle (BseStorage *self,
                                GQuark      quark)
 {
-  if (BSE_STORAGE_DBLOCK_CONTAINED (self) && quark == quark_dblock_data_handle)
+  if (BSE_STORAGE_DBLOCK_CONTAINED (self) &&
+      quark == quark_dblock_data_handle)
     return TRUE;
-  if (quark == quark_raw_data_handle)
+  if (quark == quark_raw_data_handle ||
+      quark == quark_vorbis_data_handle)
     return TRUE;
   return FALSE;
 }
@@ -1494,6 +1676,8 @@ parse_data_handle_trampoline (BseStorage     *self,
 
   if (quark == quark_raw_data_handle)
     return parse_raw_data_handle (self, data_handle_p, n_channels_p, mix_freq_p, osc_freq_p);
+  else if (quark == quark_vorbis_data_handle)
+    return parse_vorbis_data_handle (self, data_handle_p, n_channels_p, mix_freq_p, osc_freq_p);
 
   if (BSE_STORAGE_COMPAT (self, 0, 5, 1) && quark == quark_bse_storage_binary_v0)
     return compat_parse_data_handle (self, data_handle_p, n_channels_p, mix_freq_p, osc_freq_p);
@@ -1530,40 +1714,46 @@ bse_storage_parse_data_handle_rest (BseStorage     *self,
   return parse_data_handle_trampoline (self, TRUE, data_handle_p, n_channels_p, mix_freq_p, osc_freq_p);
 }
 
-void
+BseErrorType
 bse_storage_flush_fd (BseStorage *self,
                       gint        fd)
 {
-  g_return_if_fail (BSE_IS_STORAGE (self));
-  g_return_if_fail (self->wstore);
-  g_return_if_fail (fd >= 0);
+  g_return_val_if_fail (BSE_IS_STORAGE (self), BSE_ERROR_INTERNAL);
+  g_return_val_if_fail (self->wstore, BSE_ERROR_INTERNAL);
+  g_return_val_if_fail (fd >= 0, BSE_ERROR_INTERNAL);
 
   bse_storage_break (self);
 
-  sfi_wstore_flush_fd (self->wstore, fd);
+  gint nerrno = sfi_wstore_flush_fd (self->wstore, fd);
+  
+  return bse_error_from_errno (-nerrno, BSE_ERROR_FILE_WRITE_FAILED);
 }
 
-void    bse_storage_compat_dhreset      (BseStorage     *self)
+void
+bse_storage_compat_dhreset (BseStorage     *self)
 {
   self->n_channels = 1;
   self->mix_freq = 44100;
   self->osc_freq = 440;
 }
 
-void    bse_storage_compat_dhmixf       (BseStorage     *self,
-                                         gfloat          mix_freq)
+void
+bse_storage_compat_dhmixf (BseStorage     *self,
+                           gfloat          mix_freq)
 {
   self->mix_freq = mix_freq;
 }
 
-void    bse_storage_compat_dhoscf       (BseStorage     *self,
-                                         gfloat          osc_freq)
+void
+bse_storage_compat_dhoscf (BseStorage     *self,
+                           gfloat          osc_freq)
 {
   self->osc_freq = osc_freq;
 }
 
-void    bse_storage_compat_dhchannels   (BseStorage     *self,
-                                         guint           n_channels)
+void
+bse_storage_compat_dhchannels (BseStorage     *self,
+                               guint           n_channels)
 {
   self->n_channels = n_channels;
 }
@@ -1641,6 +1831,6 @@ compat_parse_data_handle (BseStorage     *self,
                                         byte_order,
                                         self->mix_freq, self->osc_freq,
                                         sfi_rstore_get_bin_offset (self->rstore) + offset,
-                                        vlength);
+                                        vlength, NULL);
   return G_TOKEN_NONE;
 }
