@@ -33,28 +33,6 @@ enum
 
 
 /* --- prototypes --- */
-static void      bse_sub_iport_init             (BseSubIPort            *self);
-static void      bse_sub_iport_class_init       (BseSubIPortClass       *class);
-static void      bse_sub_iport_set_property     (GObject                *object,
-                                                 guint                   param_id,
-                                                 const GValue           *value,
-                                                 GParamSpec             *pspec);
-static void      bse_sub_iport_get_property     (GObject                *object,
-                                                 guint                   param_id,
-                                                 GValue                 *value,
-                                                 GParamSpec             *pspec);
-static void      bse_sub_iport_finalize         (GObject                *object);
-static void      bse_sub_iport_set_parent       (BseItem                *item,
-                                                 BseItem                *parent);
-static void      bse_sub_iport_context_create   (BseSource              *source,
-                                                 guint                   instance_id,
-                                                 GslTrans               *trans);
-static void      bse_sub_iport_context_connect  (BseSource              *source,
-                                                 guint                   context_handle,
-                                                 GslTrans               *trans);
-static void      bse_sub_iport_context_dismiss  (BseSource              *source,
-                                                 guint                   context_handle,
-                                                 GslTrans               *trans);
 static void      bse_sub_iport_update_modules   (BseSubIPort            *self,
                                                  const gchar            *old_name,
                                                  const gchar            *new_name,
@@ -66,92 +44,12 @@ static gpointer          parent_class = NULL;
 
 
 /* --- functions --- */
-#include "./icons/inport.c"
-BSE_BUILTIN_TYPE (BseSubIPort)
-{
-  static const GTypeInfo type_info = {
-    sizeof (BseSubIPortClass),
-    
-    (GBaseInitFunc) NULL,
-    (GBaseFinalizeFunc) NULL,
-    (GClassInitFunc) bse_sub_iport_class_init,
-    (GClassFinalizeFunc) NULL,
-    NULL /* class_data */,
-    
-    sizeof (BseSubIPort),
-    0 /* n_preallocs */,
-    (GInstanceInitFunc) bse_sub_iport_init,
-  };
-  static const BsePixdata pixdata = {
-    INPORT_IMAGE_BYTES_PER_PIXEL | BSE_PIXDATA_1BYTE_RLE,
-    INPORT_IMAGE_WIDTH, INPORT_IMAGE_HEIGHT,
-    INPORT_IMAGE_RLE_PIXEL_DATA,
-  };
-  GType type = bse_type_register_static (BSE_TYPE_SOURCE,
-                                         "BseSubIPort",
-                                         "Virtual input port connector, used to provide a synthesis network "
-                                         "with input signals from other synthesis networks",
-                                         &type_info);
-  bse_categories_register_icon ("/Modules/Virtualization/Virtual Input", type, &pixdata);
-  return type;
-}
-
-static void
-bse_sub_iport_class_init (BseSubIPortClass *class)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
-  BseItemClass *item_class = BSE_ITEM_CLASS (class);
-  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
-  guint i, channel_id;
-  
-  parent_class = g_type_class_peek_parent (class);
-  
-  gobject_class->set_property = bse_sub_iport_set_property;
-  gobject_class->get_property = bse_sub_iport_get_property;
-  gobject_class->finalize = bse_sub_iport_finalize;
-  
-  item_class->set_parent = bse_sub_iport_set_parent;
-  
-  source_class->context_create = bse_sub_iport_context_create;
-  source_class->context_connect = bse_sub_iport_context_connect;
-  source_class->context_dismiss = bse_sub_iport_context_dismiss;
-
-  class->n_input_ports = BSE_SUB_IPORT_N_PORTS;
-
-  for (i = 0; i < class->n_input_ports; i++)
-    {
-      gchar *ident, *label, *value;
-      
-      ident = g_strdup_printf ("in_port_%u", i + 1);
-      label = g_strdup_printf (_("Input Port %u"), i + 1);
-      value = g_strdup_printf ("synth_in_%u", i + 1);
-      bse_object_class_add_param (object_class, _("Assignments"), PROP_IPORT_NAME + i * 2,
-                                  sfi_pspec_string (ident, label,
-                                                    _("The port name is a unique name to establish input<->output "
-                                                      "port relationships"),
-                                                    value, SFI_PARAM_STANDARD ":skip-default"));
-      g_free (ident);
-      g_free (label);
-      g_free (value);
-      
-      ident = g_strdup_printf ("output-%u", i + 1);
-      label = g_strdup_printf (_("Virtual input %u"), i + 1);
-      channel_id = bse_source_class_add_ochannel (source_class, ident, label, NULL);
-      g_assert (channel_id == i);
-      g_free (ident);
-      g_free (label);
-    }
-}
-
 static void
 bse_sub_iport_init (BseSubIPort *self)
 {
-  BseSubIPortClass *class = BSE_SUB_IPORT_GET_CLASS (self);
   guint i;
-
-  self->input_ports = g_new (gchar*, class->n_input_ports);
-  for (i = 0; i < class->n_input_ports; i++)
+  self->input_ports = g_new (gchar*, BSE_SOURCE_N_OCHANNELS (self));
+  for (i = 0; i < BSE_SOURCE_N_OCHANNELS (self); i++)
     self->input_ports[i] = g_strdup_printf ("synth_in_%u", i + 1);
 }
 
@@ -159,14 +57,11 @@ static void
 bse_sub_iport_finalize (GObject *object)
 {
   BseSubIPort *self = BSE_SUB_IPORT (object);
-  BseSubIPortClass *class = BSE_SUB_IPORT_GET_CLASS (self);
   guint i;
-  
-  for (i = 0; i < class->n_input_ports; i++)
+  for (i = 0; i < BSE_SOURCE_N_OCHANNELS (self); i++)
     g_free (self->input_ports[i]);
   g_free (self->input_ports);
   self->input_ports = NULL;
-
   /* chain parent class' handler */
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -188,18 +83,20 @@ bse_sub_iport_set_property (GObject      *object,
       n = (param_id - PROP_IPORT_NAME) / 2;
       switch (indx)
         {
-          const gchar *name;
         case PROP_IPORT_NAME:
-          name = g_value_get_string (value);
-          if (item->parent)
+          if (n < BSE_SOURCE_N_OCHANNELS (self))
             {
-              bse_snet_iport_name_unregister (BSE_SNET (item->parent), self->input_ports[n]);
-              name = bse_snet_iport_name_register (BSE_SNET (item->parent), name);
+              const gchar *name = g_value_get_string (value);
+              if (item->parent)
+                {
+                  bse_snet_iport_name_unregister (BSE_SNET (item->parent), self->input_ports[n]);
+                  name = bse_snet_iport_name_register (BSE_SNET (item->parent), name);
+                }
+              if (BSE_SOURCE_PREPARED (self))
+                bse_sub_iport_update_modules (self, self->input_ports[n], name, n);
+              g_free (self->input_ports[n]);
+              self->input_ports[n] = g_strdup (name);
             }
-          if (BSE_SOURCE_PREPARED (self))
-	    bse_sub_iport_update_modules (self, self->input_ports[n], name, n);
-          g_free (self->input_ports[n]);
-          self->input_ports[n] = g_strdup (name);
           break;
         default:
           G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
@@ -225,7 +122,8 @@ bse_sub_iport_get_property (GObject     *object,
       switch (indx)
         {
         case PROP_IPORT_NAME:
-          g_value_set_string (value, self->input_ports[n]);
+          if (n < BSE_SOURCE_N_OCHANNELS (self))
+            g_value_set_string (value, self->input_ports[n]);
           break;
         default:
           G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
@@ -240,12 +138,11 @@ bse_sub_iport_set_parent (BseItem *item,
                           BseItem *parent)
 {
   BseSubIPort *self = BSE_SUB_IPORT (item);
-  BseSubIPortClass *class = BSE_SUB_IPORT_GET_CLASS (self);
   guint i;
   
   /* remove port name from old parent */
   if (item->parent)
-    for (i = 0; i < class->n_input_ports; i++)
+    for (i = 0; i < BSE_SOURCE_N_OCHANNELS (self); i++)
       bse_snet_iport_name_unregister (BSE_SNET (item->parent), self->input_ports[i]);
   
   /* chain parent class' handler */
@@ -253,16 +150,14 @@ bse_sub_iport_set_parent (BseItem *item,
   
   /* add port name to new parent */
   if (item->parent)
-    for (i = 0; i < class->n_input_ports; i++)
+    for (i = 0; i < BSE_SOURCE_N_OCHANNELS (self); i++)
       {
         const gchar *name = bse_snet_iport_name_register (BSE_SNET (item->parent), self->input_ports[i]);
         if (strcmp (name, self->input_ports[i]) != 0)
           {
-            gchar *string;
-            
             g_free (self->input_ports[i]);
             self->input_ports[i] = g_strdup (name);
-            string = g_strdup_printf ("in_port_%u", i + 1);
+            gchar *string = g_strdup_printf ("in_port_%u", i + 1);
             g_object_notify (G_OBJECT (item), string);
             g_free (string);
           }
@@ -270,39 +165,41 @@ bse_sub_iport_set_parent (BseItem *item,
 }
 
 static void
-sub_iport_process (GslModule *module,
+sub_iport_process (BseModule *module,
                    guint      n_values)
 {
-  guint i, n = GSL_MODULE_N_OSTREAMS (module);
+  guint i, n = BSE_MODULE_N_OSTREAMS (module);
 
   for (i = 0; i < n; i++)
-    GSL_MODULE_OBUFFER (module, i) = (gfloat*) GSL_MODULE_IBUFFER (module, i);
+    BSE_MODULE_OBUFFER (module, i) = (gfloat*) BSE_MODULE_IBUFFER (module, i);
 }
 
 static void
 bse_sub_iport_context_create (BseSource *source,
                               guint      context_handle,
-                              GslTrans  *trans)
+                              BseTrans  *trans)
 {
-  BseSubIPortClass *class = BSE_SUB_IPORT_GET_CLASS (source);
-  GslModule *module;
+  BseSubIPort *self = BSE_SUB_IPORT (source);
 
-  if (class->gsl_class.process == NULL)
+  if (!BSE_SOURCE_GET_CLASS (self)->engine_class)
     {
-      class->gsl_class.n_istreams = class->n_input_ports;
-      class->gsl_class.n_jstreams = 0;
-      class->gsl_class.n_ostreams = class->n_input_ports;
-      class->gsl_class.process = sub_iport_process;
-      class->gsl_class.free = NULL;
-      class->gsl_class.mflags = GSL_COST_CHEAP;
+      BseModuleClass module_class = {
+        .n_istreams = BSE_SOURCE_N_OCHANNELS (self),
+        .n_jstreams = 0,
+        .n_ostreams = BSE_SOURCE_N_OCHANNELS (self),
+        .process = sub_iport_process,
+        .free = NULL,
+        .mflags = BSE_COST_CHEAP,
+      };
+      bse_source_class_cache_engine_class (BSE_SOURCE_GET_CLASS (self), &module_class);
     }
-  module = gsl_module_new (&class->gsl_class, NULL);
-  
+  BseModule *module = bse_module_new (BSE_SOURCE_GET_CLASS (self)->engine_class, NULL);
+
   /* setup module i/o streams with BseSource i/o channels */
   bse_source_set_context_omodule (source, context_handle, module);
   
   /* commit module to engine */
-  gsl_trans_add (trans, gsl_job_integrate (module));
+  bse_trans_add (trans, bse_job_integrate (module));
   
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->context_create (source, context_handle, trans);
@@ -311,18 +208,16 @@ bse_sub_iport_context_create (BseSource *source,
 static void
 bse_sub_iport_context_connect (BseSource *source,
                                guint      context_handle,
-                               GslTrans  *trans)
+                               BseTrans  *trans)
 {
   BseSubIPort *self = BSE_SUB_IPORT (source);
-  BseSubIPortClass *class = BSE_SUB_IPORT_GET_CLASS (self);
   BseItem *item = BSE_ITEM (self);
   BseSNet *snet = BSE_SNET (item->parent);
-  GslModule *module = bse_source_get_context_omodule (source, context_handle);
+  BseModule *module = bse_source_get_context_omodule (source, context_handle);
   guint i;
   
-  for (i = 0; i < class->n_input_ports; i++)
-    bse_snet_set_iport_dest (snet, self->input_ports[i], context_handle,
-                             module, i, trans);
+  for (i = 0; i < BSE_SOURCE_N_OCHANNELS (self); i++)
+    bse_snet_set_iport_dest (snet, self->input_ports[i], context_handle, module, i, trans);
   
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->context_connect (source, context_handle, trans);
@@ -331,17 +226,15 @@ bse_sub_iport_context_connect (BseSource *source,
 static void
 bse_sub_iport_context_dismiss (BseSource *source,
                                guint      context_handle,
-                               GslTrans  *trans)
+                               BseTrans  *trans)
 {
   BseSubIPort *self = BSE_SUB_IPORT (source);
-  BseSubIPortClass *class = BSE_SUB_IPORT_GET_CLASS (self);
   BseItem *item = BSE_ITEM (self);
   BseSNet *snet = BSE_SNET (item->parent);
   guint i;
   
-  for (i = 0; i < class->n_input_ports; i++)
-    bse_snet_set_iport_dest (snet, self->input_ports[i], context_handle,
-                             NULL, i, trans);
+  for (i = 0; i < BSE_SOURCE_N_OCHANNELS (self); i++)
+    bse_snet_set_iport_dest (snet, self->input_ports[i], context_handle, NULL, i, trans);
   
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->context_dismiss (source, context_handle, trans);
@@ -356,7 +249,7 @@ bse_sub_iport_update_modules (BseSubIPort   *self,
   BseItem *item = BSE_ITEM (self);
   BseSNet *snet = BSE_SNET (item->parent);
   BseSource *source = BSE_SOURCE (self);
-  GslTrans *trans = gsl_trans_open ();
+  BseTrans *trans = bse_trans_open ();
   guint *cids, n, i;
   
   g_return_if_fail (BSE_SOURCE_PREPARED (self));
@@ -364,11 +257,81 @@ bse_sub_iport_update_modules (BseSubIPort   *self,
   cids = bse_source_context_ids (source, &n);
   for (i = 0; i < n; i++)
     {
-      GslModule *module = bse_source_get_context_omodule (source, cids[i]);
-      
+      BseModule *module = bse_source_get_context_omodule (source, cids[i]);
       bse_snet_set_iport_dest (snet, old_name, cids[i], NULL, port, trans);
       bse_snet_set_iport_dest (snet, new_name, cids[i], module, port, trans);
     }
   g_free (cids);
-  gsl_trans_commit (trans);
+  bse_trans_commit (trans);
+}
+
+static void
+bse_sub_iport_class_init (BseSubIPortClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  BseItemClass *item_class = BSE_ITEM_CLASS (class);
+  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
+  guint i, channel_id;
+  
+  parent_class = g_type_class_peek_parent (class);
+  
+  gobject_class->set_property = bse_sub_iport_set_property;
+  gobject_class->get_property = bse_sub_iport_get_property;
+  gobject_class->finalize = bse_sub_iport_finalize;
+  
+  item_class->set_parent = bse_sub_iport_set_parent;
+  
+  source_class->context_create = bse_sub_iport_context_create;
+  source_class->context_connect = bse_sub_iport_context_connect;
+  source_class->context_dismiss = bse_sub_iport_context_dismiss;
+
+  for (i = 0; i < BSE_SUB_IPORT_N_PORTS; i++)
+    {
+      gchar *ident, *label, *value;
+      
+      ident = g_strdup_printf ("output-%u", i + 1);
+      label = g_strdup_printf (_("Virtual input %u"), i + 1);
+      channel_id = bse_source_class_add_ochannel (source_class, ident, label, NULL);
+      g_assert (channel_id == i);
+      g_free (ident);
+      g_free (label);
+      
+      ident = g_strdup_printf ("in_port_%u", i + 1);
+      label = g_strdup_printf (_("Input Port %u"), i + 1);
+      value = g_strdup_printf ("synth_in_%u", i + 1);
+      bse_object_class_add_param (object_class, _("Assignments"), PROP_IPORT_NAME + i * 2,
+                                  sfi_pspec_string (ident, label,
+                                                    _("The port name is a unique name to establish input<->output "
+                                                      "port relationships"),
+                                                    value, SFI_PARAM_STANDARD ":skip-default"));
+      g_free (ident);
+      g_free (label);
+      g_free (value);
+    }
+}
+
+BSE_BUILTIN_TYPE (BseSubIPort)
+{
+  static const GTypeInfo type_info = {
+    sizeof (BseSubIPortClass),
+    
+    (GBaseInitFunc) NULL,
+    (GBaseFinalizeFunc) NULL,
+    (GClassInitFunc) bse_sub_iport_class_init,
+    (GClassFinalizeFunc) NULL,
+    NULL /* class_data */,
+    
+    sizeof (BseSubIPort),
+    0 /* n_preallocs */,
+    (GInstanceInitFunc) bse_sub_iport_init,
+  };
+#include "./icons/virtual-input.c"
+  GType type = bse_type_register_static (BSE_TYPE_SOURCE,
+                                         "BseSubIPort",
+                                         "Virtual input port connector, used to provide a synthesis network "
+                                         "with input signals from other synthesis networks",
+                                         &type_info);
+  bse_categories_register_stock_module (N_("/Virtualization/Virtual Input"), type, virtual_input_pixstream);
+  return type;
 }
