@@ -333,12 +333,54 @@ track_uncross_wave (BseItem *owner,
 }
 
 static void
-clear_snet_and_wave (BseTrack *self,
-		     gboolean  need_wnet)
+create_wnet (BseTrack *self,
+	     BseWave  *wave)
+{
+  g_return_if_fail (self->wnet == NULL);
+
+  const gchar *play_type = bse_xinfos_get_value (wave->xinfos, "play-type");
+  const gchar *synthesis_network = play_type ? play_type : "wave-mono";
+
+  self->wnet = bse_project_create_intern_synth (bse_item_get_project (BSE_ITEM (self)),
+						synthesis_network,
+						BSE_TYPE_SNET);
+
+  bse_item_cross_link (BSE_ITEM (self), BSE_ITEM (self->wnet), track_uncross_wave);
+
+  if (self->sub_synth)
+    g_object_set (self->sub_synth, /* no undo */
+		  "snet", self->wnet,
+		  NULL);
+
+  if (strcmp (synthesis_network, "wave-mono") == 0)
+    {
+      g_object_set (bse_container_resolve_upath (BSE_CONTAINER (self->wnet), "wave-osc"), /* no undo */
+						 "wave", wave,
+					         NULL);
+    }
+  else if (strcmp (synthesis_network, "gus-patch") == 0)
+    {
+      g_object_set (bse_container_resolve_upath (BSE_CONTAINER (self->wnet), "wave-osc"), /* no undo */
+						 "wave", wave,
+					         NULL);
+      g_object_set (bse_container_resolve_upath (BSE_CONTAINER (self->wnet), "gus-patch-envelope"), /* no undo */
+						 "wave", wave,
+					         NULL);
+    }
+  else
+    {
+      g_warning ("track: waves with the play-type \"%s\" are not supported by this version of beast\n",
+	         synthesis_network);
+    }
+
+}
+
+static void
+clear_snet_and_wave (BseTrack *self)
 {
   g_return_if_fail (!self->sub_synth || !BSE_SOURCE_PREPARED (self->sub_synth));
 
-  if (self->sub_synth && !(self->wnet && need_wnet))
+  if (self->sub_synth)
     g_object_set (self->sub_synth, /* no undo */
 		  "snet", NULL,
 		  NULL);
@@ -356,24 +398,7 @@ clear_snet_and_wave (BseTrack *self,
       self->wave = NULL;
       g_object_notify (self, "wave");
     }
-  if (need_wnet)
-    {
-      if (!self->wnet)
-	{
-	  self->wnet = bse_project_create_intern_synth (bse_item_get_project (BSE_ITEM (self)),
-							"BSE_STD_SYNTH_MONO_WAVE",
-							BSE_TYPE_SNET);
-	  bse_item_cross_link (BSE_ITEM (self), BSE_ITEM (self->wnet), track_uncross_wave);
-	}
-      g_object_set (bse_container_resolve_upath (BSE_CONTAINER (self->wnet), "wave-osc"), /* no undo */
-		    "wave", NULL,
-		    NULL);
-      if (self->sub_synth)
-	g_object_set (self->sub_synth, /* no undo */
-		      "snet", self->wnet,
-		      NULL);
-    }
-  else if (self->wnet)
+  if (self->wnet)
     {
       BseSNet *wnet = self->wnet;
       bse_item_cross_unlink (BSE_ITEM (self), BSE_ITEM (self->wnet), track_uncross_wave);
@@ -404,7 +429,7 @@ bse_track_set_property (GObject      *object,
 	  BseSNet *snet = bse_value_get_object (value);
 	  if (snet || self->snet)
 	    {
-	      clear_snet_and_wave (self, FALSE);
+	      clear_snet_and_wave (self);
 	      self->snet = snet;
 	      if (self->snet)
 		{
@@ -424,15 +449,14 @@ bse_track_set_property (GObject      *object,
 	  BseWave *wave = bse_value_get_object (value);
 	  if (wave || self->wave)
 	    {
-	      clear_snet_and_wave (self, wave != NULL);
+	      clear_snet_and_wave (self);
+
 	      self->wave = wave;
 	      if (self->wave)
 		{
+		  create_wnet (self, wave);
 		  bse_item_cross_link (BSE_ITEM (self), BSE_ITEM (self->wave), track_uncross_wave);
 		  bse_object_proxy_notifies (self->wave, self, "changed");
-		  g_object_set (bse_container_resolve_upath (BSE_CONTAINER (self->wnet), "wave-osc"), /* no undo */
-				"wave", self->wave,
-				NULL);
 		}
 	    }
 	}
@@ -539,6 +563,7 @@ bse_track_insert_part (BseTrack *self,
       gint i = entry ? entry - self->entries_SL : 0;
       entry = track_add_entry (self, entry ? i + 1 : 0, tick, part);
     }
+  bse_part_links_changed (part);
   g_signal_emit (self, signal_changed, 0);
   return entry ? entry->id : 0;
 }
@@ -554,31 +579,29 @@ bse_track_remove_tick (BseTrack *self,
   entry = track_lookup_entry (self, tick);
   if (entry && entry->tick == tick)
     {
+      BsePart *part = entry->part;
       track_delete_entry (self, entry - self->entries_SL);
+      bse_part_links_changed (part);
       g_signal_emit (self, signal_changed, 0);
     }
 }
 
-BseTrackPartSeq*
-bse_track_list_parts (BseTrack *self)
+static BseTrackPartSeq*
+bse_track_list_parts_intern (BseTrack *self,
+                             BsePart  *part)
 {
-  BseTrackPartSeq *tps;
-  BseSongTiming timing;
+  BseItem *item = BSE_ITEM (self);
   BseSong *song = NULL;
-  BseItem *item;
-  gint i;
-
-  g_return_val_if_fail (BSE_IS_TRACK (self), NULL);
-
-  item = BSE_ITEM (self);
   if (BSE_IS_SONG (item->parent))
     song = BSE_SONG (item->parent);
+  BseSongTiming timing;
   bse_song_timing_get_default (&timing);
-  tps = bse_track_part_seq_new ();
+  BseTrackPartSeq *tps = bse_track_part_seq_new ();
+  gint i;
   for (i = 0; i < self->n_entries_SL; i++)
     {
       BseTrackEntry *entry = self->entries_SL + i;
-      if (entry->part)
+      if (entry->part && (entry->part == part || !part))
 	{
 	  BseTrackPart tp = { 0, };
 	  tp.tick = entry->tick;
@@ -594,6 +617,22 @@ bse_track_list_parts (BseTrack *self)
   return tps;
 }
 
+BseTrackPartSeq*
+bse_track_list_parts (BseTrack *self)
+{
+  g_return_val_if_fail (BSE_IS_TRACK (self), NULL);
+  return bse_track_list_parts_intern (self, NULL);
+}
+
+BseTrackPartSeq*
+bse_track_list_part (BseTrack *self,
+                     BsePart  *part)
+{
+  g_return_val_if_fail (BSE_IS_TRACK (self), NULL);
+  g_return_val_if_fail (BSE_IS_PART (part), NULL);
+  return bse_track_list_parts_intern (self, part);
+}
+
 gboolean
 bse_track_find_part (BseTrack *self,
 		     BsePart  *part,
@@ -976,7 +1015,7 @@ bse_track_class_init (BseTrackClass *class)
   bse_object_class_add_param (object_class, _("MIDI Instrument"),
                               PROP_MIDI_CHANNEL,
                               sfi_pspec_int ("midi_channel", _("MIDI Channel"),
-                                             _("Midi channel assigned to this track, 0 uses private per-track channel"),
+                                             _("Midi channel assigned to this track, 0 uses internal per-track channel"),
                                              0, 0, BSE_MIDI_MAX_CHANNELS, 1,
                                              SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale:skip-default:unprepared"));
   bse_object_class_add_param (object_class, _("MIDI Instrument"),
