@@ -451,6 +451,223 @@ bse_id_alloc (void)
 }
 
 
+/* --- string array manipulation --- */
+static gchar*
+canonify_xinfo_key (const gchar *key)
+{
+  gchar *ckey = g_strdup (key);
+  g_strcanon (ckey, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS, '-');
+  /* preserve leading dot */
+  if (key[0] == '.')
+    ckey[0] = '.';
+  return ckey;
+}
+
+gchar**
+bse_xinfos_add_value (gchar          **xinfos,
+                      const gchar     *key,
+                      const gchar     *value)
+{
+  g_return_val_if_fail (key != NULL && strchr (key, '=') == NULL, xinfos);
+  if (!value || !value[0])
+    return bse_xinfos_del_value (xinfos, key);
+  else
+    {
+      gchar *ckey = canonify_xinfo_key (key);
+      guint i, l = strlen (ckey);
+      if (xinfos)
+        {
+          for (i = 0; xinfos[i]; i++)
+            if (strncmp (xinfos[i], ckey, l) == 0 &&
+                xinfos[i][l] == '=')
+              break;
+          if (xinfos[i]) /* found value to replace */
+            {
+              g_free (xinfos[i]);
+              xinfos[i] = g_strconcat (ckey, "=", value, NULL);
+              g_free (ckey);
+              return xinfos;
+            }
+        }
+      i = xinfos ? g_strlenv (xinfos) : 0;
+      i++;
+      xinfos = g_renew (gchar*, xinfos, i + 1);
+      xinfos[i--] = NULL;
+      xinfos[i] = g_strconcat (ckey, "=", value, NULL);
+      g_free (ckey);
+      return xinfos;
+    }
+}
+
+gchar**
+bse_xinfos_parse_assignment (gchar          **xinfos,
+                             const gchar     *assignment)
+{
+  g_return_val_if_fail (assignment != NULL, xinfos);
+  const gchar *e = strchr (assignment, '=');
+  if (e && e > assignment)
+    {
+      gchar *key = g_strndup (assignment, e - assignment);
+      if (e[1]) /* key=text */
+        xinfos = bse_xinfos_add_value (xinfos, key, &e[1]);
+      else      /* key= */
+        xinfos = bse_xinfos_del_value (xinfos, key);
+    }
+  else if (!e)  /* key */
+    xinfos = bse_xinfos_del_value (xinfos, assignment);
+  return xinfos;
+}
+
+gchar**
+bse_xinfos_del_value (gchar       **xinfos,
+                      const gchar  *key)
+{
+  g_return_val_if_fail (key != NULL && strchr (key, '=') == NULL, xinfos);
+  if (xinfos)
+    {
+      gchar *ckey = canonify_xinfo_key (key);
+      guint i, l = strlen (ckey);
+      for (i = 0; xinfos[i]; i++)
+        if (strncmp (xinfos[i], ckey, l) == 0 &&
+            xinfos[i][l] == '=')
+          break;
+      g_free (ckey);
+      if (xinfos[i]) /* found value to delete */
+        {
+          g_free (xinfos[i]);
+          while (xinfos[i + 1])
+            {
+              xinfos[i] = xinfos[i + 1];
+              i++;
+            }
+          xinfos[i] = NULL;
+        }
+    }
+  return xinfos;
+}
+
+gchar**
+bse_xinfos_add_float (gchar          **xinfos,
+                      const gchar     *key,
+                      gfloat           fvalue)
+{
+  gchar buffer[G_ASCII_DTOSTR_BUF_SIZE * 2 + 1024];
+  return bse_xinfos_add_value (xinfos, key, g_ascii_dtostr (buffer, sizeof (buffer), fvalue));
+}
+
+gchar**
+bse_xinfos_add_num (gchar          **xinfos,
+                    const gchar     *key,
+                    SfiNum           num)
+{
+  gchar buffer[128];
+  g_snprintf (buffer, sizeof (buffer), "%lld", num);
+  return bse_xinfos_add_value (xinfos, key, buffer);
+}
+
+const gchar*
+bse_xinfos_get_value (gchar          **xinfos,
+                      const gchar     *key)
+{
+  g_return_val_if_fail (key != NULL && strchr (key, '=') == NULL, NULL);
+  if (xinfos)
+    {
+      guint i, l = strlen (key);
+      for (i = 0; xinfos[i]; i++)
+        if (strncmp (xinfos[i], key, l) == 0 &&
+            xinfos[i][l] == '=')
+          break;
+      if (xinfos[i]) /* found value */
+        return xinfos[i] + l + 1;
+    }
+  return NULL;
+}
+
+gfloat
+bse_xinfos_get_float (gchar          **xinfos,
+                      const gchar     *key)
+{
+  const gchar *v = bse_xinfos_get_value (xinfos, key);
+  if (v)
+    return g_ascii_strtod (v, NULL);
+  else
+    return 0.0;
+}
+
+SfiNum
+bse_xinfos_get_num (gchar          **xinfos,
+                    const gchar     *key)
+{
+  const gchar *v = bse_xinfos_get_value (xinfos, key);
+  if (v)
+    return g_ascii_strtoull (v, NULL, 10);
+  else
+    return 0.0;
+}
+
+gchar**
+bse_xinfos_dup_consolidated (gchar  **xinfos,
+                             gboolean copy_interns)
+{
+  if (xinfos)
+    {
+      /* construct list of normalized xinfos */
+      SfiRing *xinfo_list = NULL;
+      guint i = 0;
+      while (xinfos[i])
+        {
+          const gchar *xinfo = xinfos[i];
+          const gchar *e = strchr (xinfo, '=');
+          if (!e && xinfo[0])   /* empty xinfo without '=' */
+            xinfo_list = sfi_ring_append (xinfo_list, g_strconcat (xinfo, "=", NULL));
+          else if (e && !e[1])  /* empty xinfo with "=" */
+            xinfo_list = sfi_ring_append (xinfo_list, g_strdup (xinfo));
+          else if (e)           /* non-empty xinfo */
+            xinfo_list = sfi_ring_append (xinfo_list, g_strdup (xinfo));
+          i++;
+        }
+      SfiRing *rcopy = sfi_ring_copy (xinfo_list);
+      /* sort (stable, keeping order) */
+      xinfo_list = sfi_ring_sort (xinfo_list, (SfiCompareFunc) bse_xinfo_stub_compare, NULL);
+      /* remove dups (preserves first element from dup list) */
+      xinfo_list = sfi_ring_uniq_free_deep (xinfo_list, (SfiCompareFunc) bse_xinfo_stub_compare, NULL, g_free);
+      /* restore original order */
+      xinfo_list = sfi_ring_reorder (xinfo_list, rcopy);
+      sfi_ring_free (rcopy);
+      /* filter non-empty xinfos */
+      if (xinfo_list)
+        {
+          gchar **dest_xinfos = g_new (gchar*, sfi_ring_length (xinfo_list) + 1);
+          i = 0;
+          while (xinfo_list)
+            {
+              const gchar *xinfo = sfi_ring_pop_head (&xinfo_list);
+              const gchar *e = strchr (xinfo, '=');
+              if (e[1] &&       /* non-empty xinfo */
+                  (e[0] != '.' || copy_interns))
+                dest_xinfos[i++] = g_strdup (xinfo);
+            }
+          dest_xinfos[i] = NULL;
+          return dest_xinfos;
+        }
+    }
+  return NULL;
+}
+
+gint
+bse_xinfo_stub_compare (const gchar     *xinfo1,  /* must contain '=' */
+                        const gchar     *xinfo2)  /* must contain '=' */
+{
+  const gchar *e1 = strchr (xinfo1, '=');
+  gint l1 = e1 - (const gchar*) xinfo1;
+  const gchar *e2 = strchr (xinfo2, '=');
+  gint l2 = e2 - (const gchar*) xinfo2;
+  if (l1 != l2)
+    return l1 - l2;
+  return strncmp (xinfo1, xinfo2, l1);
+}
+
+
 /* --- miscellaeous --- */
 guint
 bse_string_hash (gconstpointer string)
