@@ -23,70 +23,38 @@ load_file (BseWaveRepo *wrepo,
 	   const gchar *file_name,
            BseWave    **wave_p)
 {
-  GslErrorType error;
-  GslWaveFileInfo *finfo;
-  GslWaveDsc *wdsc;
-  BseWave *wave;
-  gchar *fname;
-  guint i;
-
-  *wave_p = NULL;
-  finfo = gsl_wave_file_info_load (file_name, &error);
-  if (!finfo)
-    return error;
-  if (finfo->n_waves != 1)
-    {
-      gsl_wave_file_info_unref (finfo);
-      return BSE_ERROR_FORMAT_UNKNOWN;
-    }
-  
-  wdsc = gsl_wave_dsc_load (finfo, 0, &error);
-  gsl_wave_file_info_unref (finfo);
-  if (!wdsc)
-    return error;
-  
-  fname = g_path_get_basename (file_name);
-  wave = g_object_new (BSE_TYPE_WAVE,
-		       "uname", fname,
-		       NULL);
-  bse_wave_set_description_bits (wave, wdsc, FALSE);
+  gchar *fname = g_path_get_basename (file_name);
+  BseWave *wave = g_object_new (BSE_TYPE_WAVE, "uname", fname, NULL);
   g_free (fname);
-  for (i = 0; i < wdsc->n_chunks; i++)
-    {
-      GslWaveChunk *wchunk = gsl_wave_chunk_create (wdsc, i, error ? NULL : &error);
-      
-      if (wchunk)
-        bse_wave_add_chunk (wave, wchunk);
-    }
-  bse_wave_set_locator (wave, file_name, wdsc->name);
-  
-  gsl_wave_dsc_free (wdsc);
-  
+  BseErrorType error = bse_wave_load_wave_file (wave, file_name, NULL, NULL, NULL, TRUE);
   if (wave->n_wchunks)
     {
       bse_container_add_item (BSE_CONTAINER (wrepo), BSE_ITEM (wave));
       *wave_p = wave;
-      error = GSL_ERROR_NONE;
+      error = BSE_ERROR_NONE;
+    }
+  else
+    {
+      if (!error)
+        error = BSE_ERROR_WAVE_NOT_FOUND;
+      *wave_p = NULL;
     }
-  else if (!error)
-    error = BSE_ERROR_WAVE_NOT_FOUND;
   g_object_unref (wave);
-  
   return error;
 }
 
 
-#line 89 "bsewaverepo.proc"
+#line 57 "bsewaverepo.proc"
 
-#line 92 "bsewaverepo.proc"
+#line 60 "bsewaverepo.proc"
 
 
 /* --- load-file --- */
 static void
 load_file_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 94 "bsewaverepo.proc"
+#line 62 "bsewaverepo.proc"
  {
-#line 96 "bsewaverepo.proc"
+#line 64 "bsewaverepo.proc"
   *(in_pspecs++)    = bse_param_spec_object ("wave_repo", "Wave Repo", NULL,
 				 BSE_TYPE_WAVE_REPO, SFI_PARAM_STANDARD);
   *(in_pspecs++) = sfi_pspec_string ("file_name", "File Name", "The file to import waves from",
@@ -95,16 +63,16 @@ load_file_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **o
 				BSE_TYPE_ERROR_TYPE, BSE_ERROR_NONE,
 				SFI_PARAM_STANDARD);
 }
-#line 104 "bsewaverepo.proc"
+#line 72 "bsewaverepo.proc"
  }
 static BseErrorType
-#line 104 "bsewaverepo.proc"
+#line 72 "bsewaverepo.proc"
 load_file_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 107 "bsewaverepo.proc"
+#line 75 "bsewaverepo.proc"
 {
-#line 108 "bsewaverepo.proc"
+#line 76 "bsewaverepo.proc"
   
   BseWaveRepo *self    = (BseWaveRepo*) bse_value_get_object (in_values++);
   gchar *file_name     = sfi_value_get_string (in_values++);
@@ -112,7 +80,7 @@ load_file_exec (BseProcedureClass *proc,
   BseErrorType error;
   BseWave *wave;
   
-#line 115 "bsewaverepo.proc"
+#line 83 "bsewaverepo.proc"
   
   if (!BSE_IS_WAVE_REPO (self) || !file_name)
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -123,7 +91,7 @@ load_file_exec (BseProcedureClass *proc,
     bse_item_push_undo_proc (self, "remove-wave", wave);
   bse_item_undo_close (ustack);
   
-#line 125 "bsewaverepo.proc"
+#line 93 "bsewaverepo.proc"
   
   g_value_set_enum (out_values++, error);
   
@@ -133,47 +101,47 @@ load_file_exec (BseProcedureClass *proc,
 /* --- remove-wave --- */
 static void
 remove_wave_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 131 "bsewaverepo.proc"
+#line 99 "bsewaverepo.proc"
  {
-#line 133 "bsewaverepo.proc"
+#line 101 "bsewaverepo.proc"
   *(in_pspecs++)    = bse_param_spec_object ("wave_repo", "Wave Repo", NULL,
 				 BSE_TYPE_WAVE_REPO, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = bse_param_spec_object ("wave", "Wave", NULL,
 				 BSE_TYPE_WAVE, SFI_PARAM_STANDARD);
-#line 137 "bsewaverepo.proc"
+#line 105 "bsewaverepo.proc"
 }  }
 static BseErrorType
-#line 137 "bsewaverepo.proc"
+#line 105 "bsewaverepo.proc"
 remove_wave_exec (BseProcedureClass *proc,
 	const GValue      *in_values,
 	GValue            *out_values)
-#line 140 "bsewaverepo.proc"
+#line 108 "bsewaverepo.proc"
 {
-#line 141 "bsewaverepo.proc"
+#line 109 "bsewaverepo.proc"
   
   BseWaveRepo *self = (BseWaveRepo*) bse_value_get_object (in_values++);
   BseItem *child    = (BseItem*) bse_value_get_object (in_values++);
   BseUndoStack *ustack;
 
-#line 146 "bsewaverepo.proc"
+#line 114 "bsewaverepo.proc"
   
   if (!BSE_IS_WAVE_REPO (self) || !BSE_IS_WAVE (child) ||
       child->parent != BSE_ITEM (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 151 "bsewaverepo.proc"
+#line 119 "bsewaverepo.proc"
   
   ustack = bse_item_undo_open (self, "remove-wave %s", bse_object_debug_name (child));
-#line 153 "bsewaverepo.proc"
+#line 121 "bsewaverepo.proc"
   
   bse_container_uncross_undoable (BSE_CONTAINER (self), child);
-#line 155 "bsewaverepo.proc"
+#line 123 "bsewaverepo.proc"
   
   bse_item_push_redo_proc (self, "remove-wave", child);
-#line 157 "bsewaverepo.proc"
+#line 125 "bsewaverepo.proc"
   
   bse_container_remove_backedup (BSE_CONTAINER (self), child, ustack);
-#line 159 "bsewaverepo.proc"
+#line 127 "bsewaverepo.proc"
   
   bse_item_undo_close (ustack);
   
@@ -183,35 +151,35 @@ remove_wave_exec (BseProcedureClass *proc,
 /* --- bse-sample-file-info --- */
 static void
 bse_sample_file_info_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 165 "bsewaverepo.proc"
+#line 133 "bsewaverepo.proc"
  {
-#line 167 "bsewaverepo.proc"
+#line 135 "bsewaverepo.proc"
   *(in_pspecs++) = sfi_pspec_string ("file_name", "File Name", "The file to load info from",
 			    NULL, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_boxed ("sample_file_info", NULL, NULL, BSE_TYPE_SAMPLE_FILE_INFO, SFI_PARAM_STANDARD);
 }
-#line 171 "bsewaverepo.proc"
+#line 139 "bsewaverepo.proc"
  }
 static BseErrorType
-#line 171 "bsewaverepo.proc"
+#line 139 "bsewaverepo.proc"
 bse_sample_file_info_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 174 "bsewaverepo.proc"
+#line 142 "bsewaverepo.proc"
 {
-#line 175 "bsewaverepo.proc"
+#line 143 "bsewaverepo.proc"
   
   gchar *filename = sfi_value_get_string (in_values++);
   BseSampleFileInfo info = { 0, };
   GslWaveFileInfo *wfi = NULL;
   struct stat sbuf = { 0, };
 
-#line 181 "bsewaverepo.proc"
+#line 149 "bsewaverepo.proc"
   
   if (!filename)
     return BSE_ERROR_PROC_PARAM_INVAL;
 
-#line 185 "bsewaverepo.proc"
+#line 153 "bsewaverepo.proc"
   
   info.file = filename;
   info.waves = bse_string_seq_new ();
@@ -221,7 +189,7 @@ bse_sample_file_info_exec (BseProcedureClass *proc,
     {
       info.size = sbuf.st_size;
       info.mtime = sbuf.st_mtime * SFI_USEC_FACTOR;
-      wfi = gsl_wave_file_info_load (filename, (GslErrorType*) &info.error);
+      wfi = gsl_wave_file_info_load (filename, (BseErrorType*) &info.error);
       if (wfi)
 	{
 	  guint i;
@@ -231,11 +199,11 @@ bse_sample_file_info_exec (BseProcedureClass *proc,
 	}
     }
 
-#line 204 "bsewaverepo.proc"
+#line 172 "bsewaverepo.proc"
   
   bse_value_set_boxed (out_values++, &info);
 
-#line 207 "bsewaverepo.proc"
+#line 175 "bsewaverepo.proc"
   
   bse_string_seq_free (info.waves);
   if (wfi)
