@@ -19,8 +19,7 @@
 #ifndef __GSL_DATA_HANDLE_H__
 #define __GSL_DATA_HANDLE_H__
 
-#include <bse/gsldefs.h>
-#include <bse/gslcommon.h>	/* GslErrorType */
+#include <bse/bseutils.h>
 
 G_BEGIN_DECLS
 
@@ -33,10 +32,11 @@ G_BEGIN_DECLS
 /* --- typedefs & structures --- */
 typedef struct {
   guint		n_channels;
-  guint		bit_depth;
-  gfloat        mix_freq;
-  gfloat        osc_freq;
   GslLong	n_values;
+  gchar       **xinfos;
+  guint         bit_depth : 8;
+  guint         needs_cache : 1;
+  gfloat        mix_freq;
 } GslDataHandleSetup;
 struct _GslDataHandle
 {
@@ -47,33 +47,22 @@ struct _GslDataHandle
   SfiMutex	      mutex;
   guint		      ref_count;
   guint		      open_count;
-  GData              *qdata;
   /* opened data handle setup (open_count > 0) */
   GslDataHandleSetup  setup;
 };
 typedef void (*GslDataHandleRecurse)	(GslDataHandle		*data_handle,
 					 gpointer		 data);
-typedef enum	/*< skip >*/
-{
-  GSL_DATA_HANDLE_NEEDS_CACHE	= 1,	/* gboolean *needs_cache; */
-} GslDataHandleOJob;
 struct _GslDataHandleFuncs
 {
-  GslErrorType	(*open)			(GslDataHandle		*data_handle,
+  BseErrorType	 (*open)		(GslDataHandle		*data_handle,
 					 GslDataHandleSetup	*setup);
-  GslLong	(*read)			(GslDataHandle		*data_handle,
+  GslLong	 (*read)		(GslDataHandle		*data_handle,
 					 GslLong		 voffset, /* in values */
 					 GslLong		 n_values,
 					 gfloat			*values);
-  void		(*close)		(GslDataHandle		*data_handle);
-  void		(*recurse)		(GslDataHandle		*data_handle,
-					 GslDataHandleRecurse	 recurse,
-					 gpointer		 data);
-  void          (*destroy)		(GslDataHandle		*data_handle);
-  /* optional (for optimizations) */
-  gboolean	(*ojob)			(GslDataHandle		*data_handle,
-					 GslDataHandleOJob	 ojob,
-					 gpointer		 data);
+  void		 (*close)		(GslDataHandle		*data_handle);
+  GslDataHandle* (*get_source)          (GslDataHandle          *data_handle);
+  void           (*destroy)		(GslDataHandle		*data_handle);
 };
 
 
@@ -81,11 +70,7 @@ struct _GslDataHandleFuncs
 /* --- standard functions --- */
 GslDataHandle*	  gsl_data_handle_ref		(GslDataHandle	  *dhandle);
 void		  gsl_data_handle_unref		(GslDataHandle	  *dhandle);
-void    	  gsl_data_handle_override	(GslDataHandle	  *dhandle,
-                                                 gint              bit_depth,
-                                                 gfloat            mix_freq,
-                                                 gfloat            osc_freq);
-GslErrorType	  gsl_data_handle_open		(GslDataHandle	  *dhandle);
+BseErrorType	  gsl_data_handle_open		(GslDataHandle	  *dhandle);
 void		  gsl_data_handle_close		(GslDataHandle	  *dhandle);
 GslLong		  gsl_data_handle_length	(GslDataHandle	  *data_handle);
 #define	          gsl_data_handle_n_values(      dh) \
@@ -99,6 +84,7 @@ GslLong		  gsl_data_handle_read		(GslDataHandle	  *data_handle,
 						 GslLong	   value_offset,
 						 GslLong	   n_values,
 						 gfloat		  *values);
+GslDataHandle*    gsl_data_handle_get_source    (GslDataHandle    *dhandle);
 GslDataHandle*	  gsl_data_handle_new_cut	(GslDataHandle	  *src_handle,
 						 GslLong	   cut_offset,
 						 GslLong	   n_cut_values);
@@ -125,6 +111,13 @@ GslDataHandle*	  gsl_data_handle_new_looped	(GslDataHandle	  *src_handle,
 						 GslLong	   loop_first,
 						 GslLong	   loop_last);
 
+/* --- xinfo handling --- */
+GslDataHandle* gsl_data_handle_new_add_xinfos     (GslDataHandle *src_handle,
+                                                   gchar        **xinfos);
+GslDataHandle* gsl_data_handle_new_remove_xinfos  (GslDataHandle *src_handle,
+                                                   gchar        **xinfos);
+GslDataHandle* gsl_data_handle_new_clear_xinfos   (GslDataHandle *src_handle);
+
 
 /* --- wave specific functions --- */
 typedef enum    /*< skip >*/
@@ -132,6 +125,8 @@ typedef enum    /*< skip >*/
   GSL_WAVE_FORMAT_NONE,
   GSL_WAVE_FORMAT_UNSIGNED_8,
   GSL_WAVE_FORMAT_SIGNED_8,
+  GSL_WAVE_FORMAT_ALAW,
+  GSL_WAVE_FORMAT_ULAW,
   GSL_WAVE_FORMAT_UNSIGNED_12,
   GSL_WAVE_FORMAT_SIGNED_12,
   GSL_WAVE_FORMAT_UNSIGNED_16,
@@ -139,6 +134,7 @@ typedef enum    /*< skip >*/
   GSL_WAVE_FORMAT_FLOAT,
   GSL_WAVE_FORMAT_LAST
 } GslWaveFormatType;
+#define GSL_WAVE_FORMAT_IS_LAW(f)       ((f) == GSL_WAVE_FORMAT_ALAW || (f) == GSL_WAVE_FORMAT_ULAW)
 
 const gchar*      gsl_wave_format_to_string     (GslWaveFormatType format);
 GslWaveFormatType gsl_wave_format_from_string   (const gchar      *string);
@@ -149,7 +145,8 @@ GslDataHandle*	  gsl_wave_handle_new		(const gchar	  *file_name,
                                                  gfloat            mix_freq,
                                                  gfloat            osc_freq,
 						 GslLong	   byte_offset,
-						 GslLong	   n_values);
+						 GslLong	   n_values,
+                                                 gchar           **xinfos);
 GslDataHandle*	  gsl_wave_handle_new_zoffset	(const gchar	  *file_name,
 						 guint		   n_channels,
 						 GslWaveFormatType format,
@@ -157,7 +154,8 @@ GslDataHandle*	  gsl_wave_handle_new_zoffset	(const gchar	  *file_name,
                                                  gfloat            mix_freq,
                                                  gfloat            osc_freq,
 						 GslLong	   byte_offset,
-						 GslLong	   byte_size);
+						 GslLong	   byte_size,
+                                                 gchar           **xinfos);
 guint		  gsl_wave_format_bit_depth	(GslWaveFormatType format);
 guint		  gsl_wave_format_byte_width	(GslWaveFormatType format);
 
