@@ -1,5 +1,5 @@
 /* GSL - Generic Sound Layer
- * Copyright (C) 2000-2002 Tim Janik
+ * Copyright (C) 2000-2005 Tim Janik
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,13 +33,13 @@ typedef struct
   GslWaveFileInfo wfi;
   OggVorbis_File  ofile;
 } FileInfo;
-
+#define LOADER_LOGICAL_BIT_STREAM(chunk)    ((chunk).loader_data[0].uint)
 
 /* --- functions --- */
 static GslWaveFileInfo*
 oggv_load_file_info (gpointer      data,
 		     const gchar  *file_name,
-		     GslErrorType *error_p)
+		     BseErrorType *error_p)
 {
   FileInfo *fi = sfi_new_struct0 (FileInfo, 1);
   FILE *file;
@@ -48,7 +48,7 @@ oggv_load_file_info (gpointer      data,
   file = fopen (file_name, "r");
   if (!file)
     {
-      *error_p = gsl_error_from_errno (errno, GSL_ERROR_OPEN_FAILED);
+      *error_p = gsl_error_from_errno (errno, BSE_ERROR_FILE_OPEN_FAILED);
       return NULL;
     }
   
@@ -58,7 +58,7 @@ oggv_load_file_info (gpointer      data,
     {
       fclose (file);
       sfi_delete_struct (FileInfo, fi);
-      *error_p = GSL_ERROR_CODEC_FAILURE;
+      *error_p = BSE_ERROR_CODEC_FAILURE;
       return NULL;
     }
 
@@ -99,7 +99,7 @@ static GslWaveDsc*
 oggv_load_wave_dsc (gpointer         data,
 		    GslWaveFileInfo *file_info,
 		    guint            nth_wave,
-		    GslErrorType    *error_p)
+		    BseErrorType    *error_p)
 {
   FileInfo *fi = (FileInfo*) file_info;
   GslWaveDsc *wdsc = sfi_new_struct0 (GslWaveDsc, 1);
@@ -111,7 +111,7 @@ oggv_load_wave_dsc (gpointer         data,
   wdsc->chunks = g_new0 (GslWaveChunkDsc, 1);
   wdsc->chunks[0].osc_freq = 440.0; /* FIXME */
   wdsc->chunks[0].mix_freq = vi->rate;
-  wdsc->chunks[0].loader_offset = nth_wave;	/* lbitstream */
+  LOADER_LOGICAL_BIT_STREAM (wdsc->chunks[0]) = nth_wave;
 
   return wdsc;
 }
@@ -120,8 +120,11 @@ static void
 oggv_free_wave_dsc (gpointer    data,
 		    GslWaveDsc *wdsc)
 {
-  g_free (wdsc->name);
+  guint i;
+  for (i = 0; i < wdsc->n_chunks; i++)
+    g_strfreev (wdsc->chunks[i].xinfos);
   g_free (wdsc->chunks);
+  g_free (wdsc->name);
   sfi_delete_struct (GslWaveDsc, wdsc);
 }
 
@@ -129,7 +132,7 @@ static GslDataHandle*
 oggv_create_chunk_handle (gpointer      data,
 			  GslWaveDsc   *wdsc,
 			  guint         nth_chunk,
-			  GslErrorType *error_p)
+			  BseErrorType *error_p)
 {
   FileInfo *fi = (FileInfo*) wdsc->file_info;
   GslDataHandle *dhandle;
@@ -137,10 +140,16 @@ oggv_create_chunk_handle (gpointer      data,
   g_return_val_if_fail (nth_chunk == 0, NULL);
 
   dhandle = gsl_data_handle_new_ogg_vorbis_muxed (fi->wfi.file_name,
-					          wdsc->chunks[0].loader_offset, /* lbitstream */
+					          LOADER_LOGICAL_BIT_STREAM (wdsc->chunks[0]),
 					          wdsc->chunks[0].osc_freq);
+  if (dhandle && wdsc->chunks[0].xinfos)
+    {
+      GslDataHandle *tmp_handle = dhandle;
+      dhandle = gsl_data_handle_new_add_xinfos (dhandle, wdsc->chunks[0].xinfos);
+      gsl_data_handle_unref (tmp_handle);
+    }
   if (!dhandle)
-    *error_p = GSL_ERROR_OPEN_FAILED;
+    *error_p = BSE_ERROR_FILE_OPEN_FAILED;
   return dhandle;
 }
 
@@ -148,7 +157,7 @@ void
 _gsl_init_loader_oggvorbis (void)
 {
   static const gchar *file_exts[] = { "ogg", NULL, };
-  static const gchar *mime_types[] = { "application/x-ogg", "audio/x-vorbis", "audio/x-ogg", NULL, };
+  static const gchar *mime_types[] = { "application/ogg", "application/x-ogg", "audio/x-vorbis", "audio/x-ogg", NULL, };
   static const gchar *magics[] = { "0 string OggS\n" "29 string vorbis", NULL, };
   static GslLoader loader = {
     "Ogg/Vorbis",
