@@ -1,5 +1,5 @@
 /* GSL - Generic Sound Layer
- * Copyright (C) 2001-2004 Tim Janik
+ * Copyright (C) 2001-2005 Tim Janik
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -148,14 +148,14 @@ gsl_loader_match (const gchar *file_name)
 
 GslWaveFileInfo*
 gsl_wave_file_info_load (const gchar  *file_name,
-			 GslErrorType *error_p)
+			 BseErrorType *error_p)
 {
   GslWaveFileInfo *finfo = NULL;
-  GslErrorType error = GSL_ERROR_NONE;
+  BseErrorType error = BSE_ERROR_NONE;
   GslLoader *loader;
   
   if (error_p)
-    *error_p = GSL_ERROR_INTERNAL;
+    *error_p = BSE_ERROR_INTERNAL;
   g_return_val_if_fail (file_name != NULL, NULL);
 
   loader = gsl_loader_match (file_name);
@@ -169,7 +169,7 @@ gsl_wave_file_info_load (const gchar  *file_name,
 	  finfo = NULL;
 	}
       if (!finfo && !error)
-	error = GSL_ERROR_FILE_EMPTY;	/* FIXME: try next loader */
+	error = BSE_ERROR_FILE_EMPTY;	/* FIXME: try next loader */
       if (finfo)
 	{
 	  if (finfo->n_waves > 0)
@@ -190,7 +190,7 @@ gsl_wave_file_info_load (const gchar  *file_name,
 	    {
 	      loader->free_file_info (loader->data, finfo);
 	      finfo = NULL;
-	      error = GSL_ERROR_FILE_EMPTY;   /* FIXME: try next loader */
+	      error = BSE_ERROR_FILE_EMPTY;   /* FIXME: try next loader */
 	    }
 	}
     }
@@ -199,7 +199,7 @@ gsl_wave_file_info_load (const gchar  *file_name,
       /* try to provide apropriate error code */
       error = gsl_file_check (file_name, "rf");
       if (!error)
-	error = GSL_ERROR_FORMAT_UNKNOWN;
+	error = BSE_ERROR_FORMAT_UNKNOWN;
     }
 
   if (error_p)
@@ -222,6 +222,8 @@ gsl_wave_file_info_unref (GslWaveFileInfo *wave_file_info)
       g_free (wave_file_info->file_name);
       wave_file_info->file_name = NULL;
       wave_file_info->loader = NULL;
+      g_strfreev (wave_file_info->comments);
+      wave_file_info->comments = NULL;
 
       loader->free_file_info (loader->data, wave_file_info);
     }
@@ -249,14 +251,15 @@ gsl_wave_file_info_loader (GslWaveFileInfo *fi)
 GslWaveDsc*
 gsl_wave_dsc_load (GslWaveFileInfo *wave_file_info,
 		   guint            nth_wave,
-		   GslErrorType    *error_p)
+                   gboolean         accept_empty,
+                   BseErrorType    *error_p)
 {
-  GslErrorType error = GSL_ERROR_NONE;
+  BseErrorType error = BSE_ERROR_NONE;
   GslWaveDsc *wdsc;
   GslLoader *loader;
 
   if (error_p)
-    *error_p = GSL_ERROR_INTERNAL;
+    *error_p = BSE_ERROR_INTERNAL;
   g_return_val_if_fail (wave_file_info != NULL, NULL);
   g_return_val_if_fail (wave_file_info->loader != NULL, NULL);
   g_return_val_if_fail (nth_wave < wave_file_info->n_waves, NULL);
@@ -271,10 +274,10 @@ gsl_wave_dsc_load (GslWaveFileInfo *wave_file_info,
       wdsc = NULL;
     }
   if (!wdsc && !error)
-    error = GSL_ERROR_FILE_EMPTY;
+    error = BSE_ERROR_FILE_EMPTY;
   if (wdsc)
     {
-      if (wdsc->n_chunks > 0)
+      if (accept_empty || wdsc->n_chunks > 0)
 	{
 	  g_return_val_if_fail (wdsc->file_info == NULL, NULL);
 	  g_return_val_if_fail (wdsc->name && strcmp (wdsc->name, wave_file_info->waves[nth_wave].name) == 0, NULL);
@@ -286,7 +289,7 @@ gsl_wave_dsc_load (GslWaveFileInfo *wave_file_info,
 	{
 	  loader->free_wave_dsc (loader->data, wdsc);
 	  wdsc = NULL;
-	  error = GSL_ERROR_FILE_EMPTY;
+	  error = BSE_ERROR_FILE_EMPTY;
 	}
     }
 
@@ -299,14 +302,11 @@ gsl_wave_dsc_load (GslWaveFileInfo *wave_file_info,
 void
 gsl_wave_dsc_free (GslWaveDsc *wave_dsc)
 {
-  GslWaveFileInfo *file_info;
-
   g_return_if_fail (wave_dsc != NULL);
   g_return_if_fail (wave_dsc->file_info != NULL);
 
-  file_info = wave_dsc->file_info;
-  wave_dsc->file_info = NULL;
-  
+  GslWaveFileInfo *file_info = wave_dsc->file_info;
+
   file_info->loader->free_wave_dsc (file_info->loader->data, wave_dsc);
 
   gsl_wave_file_info_unref (file_info);
@@ -315,14 +315,14 @@ gsl_wave_dsc_free (GslWaveDsc *wave_dsc)
 GslDataHandle*
 gsl_wave_handle_create (GslWaveDsc   *wave_dsc,
 			guint	      nth_chunk,
-			GslErrorType *error_p)
+			BseErrorType *error_p)
 {
-  GslErrorType error = GSL_ERROR_NONE;
+  BseErrorType error = BSE_ERROR_NONE;
   GslDataHandle *dhandle;
   GslLoader *loader;
 
   if (error_p)
-    *error_p = GSL_ERROR_INTERNAL;
+    *error_p = BSE_ERROR_INTERNAL;
   g_return_val_if_fail (wave_dsc != NULL, NULL);
   g_return_val_if_fail (wave_dsc->file_info != NULL, NULL);
   g_return_val_if_fail (nth_chunk < wave_dsc->n_chunks, NULL);
@@ -340,7 +340,7 @@ gsl_wave_handle_create (GslWaveDsc   *wave_dsc,
       dhandle = NULL;
     }
   if (!dhandle && !error)
-    error = GSL_ERROR_FORMAT_INVALID;
+    error = BSE_ERROR_FORMAT_INVALID;
 
   if (error_p)
     *error_p = error;
@@ -351,14 +351,14 @@ gsl_wave_handle_create (GslWaveDsc   *wave_dsc,
 GslWaveChunk*
 gsl_wave_chunk_create (GslWaveDsc   *wave_dsc,
 		       guint         nth_chunk,
-		       GslErrorType *error_p)
+		       BseErrorType *error_p)
 {
   GslDataHandle *dhandle;
   GslDataCache *dcache;
   GslWaveChunk *wchunk;
 
   if (error_p)
-    *error_p = GSL_ERROR_INTERNAL;
+    *error_p = BSE_ERROR_INTERNAL;
   g_return_val_if_fail (wave_dsc != NULL, NULL);
   g_return_val_if_fail (nth_chunk < wave_dsc->n_chunks, NULL);
 
@@ -366,8 +366,10 @@ gsl_wave_chunk_create (GslWaveDsc   *wave_dsc,
   if (!dhandle)
     return NULL;
 
+  GslWaveChunkDsc *chunk = wave_dsc->chunks + nth_chunk;
+
   if (error_p)
-    *error_p = GSL_ERROR_IO;
+    *error_p = BSE_ERROR_IO;
 
   /* FIXME: we essentially create a dcache for each wchunk here ;( */
 
@@ -377,17 +379,26 @@ gsl_wave_chunk_create (GslWaveDsc   *wave_dsc,
     return NULL;
   /* dcache keeps dhandle alive */
 
-  wchunk = gsl_wave_chunk_new (dcache,
-			       wave_dsc->chunks[nth_chunk].mix_freq,
-			       wave_dsc->chunks[nth_chunk].osc_freq,
-			       wave_dsc->chunks[nth_chunk].loop_type,
-			       wave_dsc->chunks[nth_chunk].loop_start,
-			       wave_dsc->chunks[nth_chunk].loop_end,
-			       wave_dsc->chunks[nth_chunk].loop_count);
+  const gchar *ltype = bse_xinfos_get_value (chunk->xinfos, "loop-type");
+  GslWaveLoopType loop_type = ltype ? gsl_wave_loop_type_from_string (ltype) : GSL_WAVE_LOOP_NONE;
+  SfiNum loop_start = bse_xinfos_get_num (chunk->xinfos, "loop-start");
+  SfiNum loop_end = bse_xinfos_get_num (chunk->xinfos, "loop-end");
+  SfiNum loop_count = bse_xinfos_get_num (chunk->xinfos, "loop-count");
+  if (loop_type && loop_count == 0)     /* loop_count defaults to maximum */
+    loop_count = 1000000;
+  if (loop_end <= loop_start)
+    {
+      loop_start = loop_end = 0;
+      loop_type = GSL_WAVE_LOOP_NONE;
+      loop_count = 0;
+    }
+
+  wchunk = gsl_wave_chunk_new (dcache, chunk->mix_freq, chunk->osc_freq,
+                               loop_type, loop_start, loop_end, loop_count);
   gsl_data_cache_unref (dcache);
 
   if (error_p && wchunk)
-    *error_p = GSL_ERROR_NONE;
+    *error_p = BSE_ERROR_NONE;
 
   return wchunk;
 }
