@@ -1,5 +1,5 @@
 /* GSL - Generic Sound Layer
- * Copyright (C) 2001-2004 Tim Janik
+ * Copyright (C) 2001-2005 Tim Janik
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,8 +19,7 @@
 #ifndef __GSL_LOADER_H__
 #define __GSL_LOADER_H__
 
-#include <bse/gsldefs.h>
-#include <bse/gslcommon.h>
+#include <bse/bseutils.h>
 #include <bse/gslwavechunk.h>
 
 G_BEGIN_DECLS
@@ -33,6 +32,8 @@ struct _GslWaveFileInfo
     gchar *name;
   }       *waves;
 
+  gchar  **comments;
+
   /*< private >*/
   gchar     *file_name;
   GslLoader *loader;
@@ -44,50 +45,47 @@ struct _GslWaveDsc
   guint	           n_chunks;
   GslWaveChunkDsc *chunks;
   guint            n_channels;
-  gchar           *authors;
-  gchar           *license;
-  gchar           *comment;
+  gchar          **xinfos;
   /*< private >*/
   GslWaveFileInfo *file_info;
 };
 struct _GslWaveChunkDsc
 {
-  gfloat	  mix_freq;
   gfloat	  osc_freq;
-  GslWaveLoopType loop_type;
-  GslLong	  loop_start;	/* sample offset */
-  GslLong	  loop_end;	/* sample offset */
-  guint		  loop_count;
+  gfloat	  mix_freq;
+  gchar         **xinfos;
   /* loader-specific */
-  GslLong         loader_offset;
-  GslLong         loader_length;
-  glong		  loader_num1;
-  gpointer	  loader_data1; /* generic pointers for more data */
-  gpointer	  loader_data2;
+  union {
+    guint         uint;
+    gpointer      ptr;
+    gfloat        vfloat;
+  }               loader_data[8];
 };
 
 
 /* --- functions --- */
 GslWaveFileInfo*      gsl_wave_file_info_load	(const gchar	 *file_name,
-						 GslErrorType	 *error);
+						 BseErrorType	 *error);
 GslWaveFileInfo*      gsl_wave_file_info_ref	(GslWaveFileInfo *wave_file_info);
 void                  gsl_wave_file_info_unref	(GslWaveFileInfo *wave_file_info);
 const gchar*	      gsl_wave_file_info_loader	(GslWaveFileInfo *fi);
 GslWaveDsc*	      gsl_wave_dsc_load		(GslWaveFileInfo *wave_file_info,
 						 guint		  nth_wave,
-						 GslErrorType	 *error);
+                                                 gboolean         accept_empty,
+						 BseErrorType	 *error);
 void		      gsl_wave_dsc_free		(GslWaveDsc	 *wave_dsc);
 GslDataHandle*	      gsl_wave_handle_create	(GslWaveDsc	 *wave_dsc,
 						 guint		  nth_chunk,
-						 GslErrorType	 *error);
+						 BseErrorType	 *error);
 GslWaveChunk*	      gsl_wave_chunk_create	(GslWaveDsc	 *wave_dsc,
 						 guint		  nth_chunk,
-						 GslErrorType	 *error);
+						 BseErrorType	 *error);
 
 
 /* --- loader impl --- */
 typedef enum /*< skip >*/
 {
+  GSL_LOADER_NO_FLAGS              = 0,
   GSL_LOADER_SKIP_PRECEEDING_NULLS = 1 << 0
 } GslLoaderFlags;
 struct _GslLoader
@@ -101,7 +99,7 @@ struct _GslLoader
   const gchar **extensions;	/* e.g.: "mp3", "ogg" or "bsewave" */
   const gchar **mime_types;	/* e.g.: "audio/x-mpg3" or "audio/x-wav" */
   GslLoaderFlags flags;
-  const gchar **magic_specs;	/* e.g.: "0 string RIFF\n8 string WAVE\n" or "0 string #BseWave\n" */
+  const gchar **magic_specs;	/* e.g.: "0 string RIFF\n8 string WAVE\n" or "0 string #BseWave1\n" */
 
   gint   priority;   /* -100=high, +100=low, 0=default */
 
@@ -109,24 +107,24 @@ struct _GslLoader
   gpointer		  data;
   GslWaveFileInfo*	(*load_file_info)	(gpointer	   data,
 						 const gchar	  *file_name,
-						 GslErrorType	  *error);
+						 BseErrorType	  *error);
   void			(*free_file_info)	(gpointer	   data,
 						 GslWaveFileInfo  *file_info);
   GslWaveDsc*		(*load_wave_dsc)	(gpointer	   data,
 						 GslWaveFileInfo  *file_info,
 						 guint		   nth_wave,
-						 GslErrorType	  *error);
+						 BseErrorType	  *error);
   void			(*free_wave_dsc)	(gpointer	   data,
 						 GslWaveDsc	  *wave_dsc);
   GslDataHandle*	(*create_chunk_handle)	(gpointer	   data,
 						 GslWaveDsc	  *wave_dsc,
 						 guint		   nth_chunk,
-						 GslErrorType	  *error);
+						 BseErrorType	  *error);
   GslLoader   *next;	/* must be NULL */
 };
 
-void		gsl_loader_register	(GslLoader	*loader);
-GslLoader*	gsl_loader_match	(const gchar	*file_name);
+void	      gsl_loader_register	        (GslLoader	 *loader);
+GslLoader*    gsl_loader_match	                (const gchar	 *file_name);
 
 G_END_DECLS
 
