@@ -84,22 +84,13 @@ main (int   argc,
  restart:
   a = b = 0;
   
-  if (strcmp (arg, "approx-exp2-run") == 0)
+  if (strcmp (arg, "approx5-exp2-run") == 0)
     {
       gfloat f;
       gdouble r = 0;
       for (i = 0; i < 10; i++)
 	for (f = -1; f <= 1.0; f += 1.0 / 10000000.0)
-	  r += bse_approx_exp2 (f);
-      return (r > 0) & 8;
-    }
-  else if (strcmp (arg, "old-exp2-run") == 0)
-    {
-      gfloat f;
-      gdouble r = 0;
-      for (i = 0; i < 10; i++)
-	for (f = -1; f <= 1.0; f += 1.0 / 10000000.0)
-	  r += _bse_signal_exp2_fraction (f); // bse_signal_exp2 (f);
+	  r += bse_approx5_exp2 (f);
       return (r > 0) & 8;
     }
   else if (strcmp (arg, "libc-exp-run") == 0)
@@ -118,7 +109,7 @@ main (int   argc,
       while (file)
 	{
 	  GslWaveFileInfo *fi;
-	  GslErrorType error;
+	  BseErrorType error;
 	  
 	  fi = gsl_wave_file_info_load (file, &error);
 	  if (fi)
@@ -131,7 +122,7 @@ main (int   argc,
 	      gsl_wave_file_info_unref (fi);
 	    }
 	  else
-	    g_print ("Failed to scan \"%s\": %s\n", file, gsl_strerror (error));
+	    g_print ("Failed to scan \"%s\": %s\n", file, bse_error_blurb (error));
 	  file = pshift ();
 	  if (!file[0])
 	    break;
@@ -142,12 +133,12 @@ main (int   argc,
       const gchar *file = pshift ();
       
       g_print ("file test for \"%s\":\n", file);
-      g_print ("  is readable   : %s\n", gsl_strerror (gsl_file_check (file, "r")));
-      g_print ("  is writable   : %s\n", gsl_strerror (gsl_file_check (file, "w")));
-      g_print ("  is executable : %s\n", gsl_strerror (gsl_file_check (file, "x")));
-      g_print ("  is file       : %s\n", gsl_strerror (gsl_file_check (file, "f")));
-      g_print ("  is directory  : %s\n", gsl_strerror (gsl_file_check (file, "d")));
-      g_print ("  is link       : %s\n", gsl_strerror (gsl_file_check (file, "l")));
+      g_print ("  is readable   : %s\n", bse_error_blurb (gsl_file_check (file, "r")));
+      g_print ("  is writable   : %s\n", bse_error_blurb (gsl_file_check (file, "w")));
+      g_print ("  is executable : %s\n", bse_error_blurb (gsl_file_check (file, "x")));
+      g_print ("  is file       : %s\n", bse_error_blurb (gsl_file_check (file, "f")));
+      g_print ("  is directory  : %s\n", bse_error_blurb (gsl_file_check (file, "d")));
+      g_print ("  is link       : %s\n", bse_error_blurb (gsl_file_check (file, "l")));
     }
   else if (strcmp (arg, "ring-test") == 0)
     {
