@@ -16,7 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include "davchorus.gen-idl.h"
+#include "davchorus.genidl.hh"
 
 namespace Bse {
 namespace Dav {
