@@ -16,9 +16,9 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include "sfidl-corecxx.h"
-#include "sfidl-namespace.h"
-#include "sfidl-factory.h"
+#include "sfidl-corecxx.hh"
+#include "sfidl-namespace.hh"
+#include "sfidl-factory.hh"
 
 #include <string.h>
 #include <stdio.h>
@@ -487,7 +487,7 @@ public:
           continue;
         nspace.setFromSymbol(ci->name);
         const char *name = nspace.printable_form (ci->name);
-        printf ("static inline SfiChoiceValues %s_choice_values();\n", name);
+        printf ("static SfiChoiceValues %s_choice_values();\n", name);
       }
   }
   void
@@ -500,7 +500,7 @@ public:
           continue;
         nspace.setFromSymbol(ci->name);
         const char *name = nspace.printable_form (ci->name);
-        printf ("static inline SfiChoiceValues\n");
+        printf ("static SfiChoiceValues\n");
         printf ("%s_choice_values()\n", name);
         printf ("{\n");
         printf ("  static SfiChoiceValue values[%u];\n", ci->contents.size());
@@ -1009,7 +1009,7 @@ public:
             printf ("    break;\n");
           }
         printf ("    };\n");
-        printf ("    property_changed ((%s) prop_id);\n", ctPropertyID);
+        printf ("    property_changed (%s (prop_id));\n", ctPropertyID);
         printf ("    update_modules();\n");
         /* reset triggers */
         printf ("    switch (prop_id) {\n");
@@ -1039,6 +1039,7 @@ public:
         /* property_updated() */
         printf ("  void property_updated (%s prop_id, guint64 tick_stamp, double prop_value, GParamSpec *pspec)\n", ctPropertyID);
         printf ("  {\n");
+        printf ("    bool seen_change = false;\n");
         printf ("    switch (prop_id) {\n");
         for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
           {
@@ -1046,12 +1047,18 @@ public:
               continue;
             printf ("    case PROP_%s:\n", pure_UPPER (pi->name));
             printf ("      if (tick_stamp >= ::std::max (last__%s, module_update_tick_stamp()))\n", pi->name.c_str());
-            printf ("        %s = prop_value;\n", pi->name.c_str());
+            printf ("        {\n");
+            printf ("          seen_change = true;\n");
+            printf ("          %s = prop_value;\n", pi->name.c_str());
+            printf ("        }\n");
             printf ("      last__%s = tick_stamp;\n", pi->name.c_str());
             printf ("    break;\n");
           }
         printf ("    default: ;\n");
         printf ("    };\n");
+        printf ("    if (seen_change &&\n");
+        printf ("        property_changed (%s (prop_id)))\n", ctPropertyID);
+        printf ("      update_modules();\n");
         printf ("  }\n");
 
         /* static data */
@@ -1067,7 +1074,7 @@ public:
         
         /* property-changed hooking */
         printf ("protected:\n");
-        printf ("  virtual void property_changed (%s) {}\n", ctPropertyID);
+        printf ("  virtual bool property_changed (%s) { return false; }\n", ctPropertyID);
         
         /* methods */
         for (vector<Method>::const_iterator mi = ci->methods.begin(); mi != ci->methods.end(); mi++)
@@ -1292,7 +1299,7 @@ public:
     printf ("\n/*-------- begin %s generated code --------*/\n\n\n", Options::the()->sfidlName.c_str());
     
     /* standard includes */
-    printf ("\n#include <bse/bsecxxplugin.h>\n");
+    printf ("\n#include <bse/bsecxxplugin.hh>\n");
     
     /* reset auxillary structures */
     images.resize (0);
